@@ -48,6 +48,10 @@ const (
 
 	// Event size - https://docs.aws.amazon.com/AmazonCloudWatch/latest/logs/cloudwatch_limits_cwl.html
 	MessageLengthThresholdInBytes = 200 * 1000
+	// json.Marshal can inflate streamed event message by up to ~6 times the size of the message, depending on message contents.
+	// Threshold is reduced here to 1/6 size to account for this.
+	// https://go.dev/play/p/G3RalE_BUEL
+	StreamMessageLengthThresholdInBytes = 33000
 )
 
 // CloudWatchLogsService encapsulates the client and stop policy as a wrapper to call the cloudwatchlogs API
@@ -95,13 +99,13 @@ func createCloudWatchStopPolicy() *sdkutil.StopPolicy {
 
 // createCloudWatchClient creates a client to call CloudWatchLogs APIs
 func createCloudWatchClient(context context.T) cloudwatchlogsinterface.CloudWatchLogsClient {
-	config := sdkutil.AwsConfig(context)
+	config := sdkutil.AwsConfig(context, "logs")
 	return createCloudWatchClientWithConfig(context, config)
 }
 
 // createCloudWatchClientWithCredentials creates a client to call CloudWatchLogs APIs using credentials from the id and secret passed
 func createCloudWatchClientWithCredentials(context context.T, id, secret string) cloudwatchlogsinterface.CloudWatchLogsClient {
-	config := sdkutil.AwsConfig(context).WithCredentials(credentials.NewStaticCredentials(id, secret, ""))
+	config := sdkutil.AwsConfig(context, "logs").WithCredentials(credentials.NewStaticCredentials(id, secret, ""))
 	return createCloudWatchClientWithConfig(context, config)
 }
 
@@ -112,10 +116,6 @@ func createCloudWatchClientWithConfig(context context.T, config *aws.Config) clo
 		NumMaxRetries: maxRetries,
 	})
 
-	if defaultEndpoint := context.Identity().GetDefaultEndpoint("logs"); defaultEndpoint != "" {
-		config.Endpoint = &defaultEndpoint
-	}
-
 	appConfig := context.AppConfig()
 
 	sess := session.New(config)
@@ -192,21 +192,26 @@ func (service *CloudWatchLogsService) CreateLogGroup(logGroup string) (err error
 
 	//Calling the API
 	if _, err = service.cloudWatchLogsClient.CreateLogGroup(params); err != nil {
-		// Handle the common AWS errors and update the stop policy accordingly
-		sdkutil.HandleAwsError(log, err, service.stopPolicy)
-
 		// Cast err to awserr.Error to get the Code
 		errorCode := sdkutil.GetAwsErrorCode(err)
 
 		switch errorCode {
 		// Check for error code. Note that the AWS Retrier has already made retries for the 5xx Response Codes
 		case resourceAlreadyExistsException:
+			log.Debugf("log group /%v/ already exists", logGroup)
 			// 400 Error, occurs when the LogGroup already exists
 			// Ignoring the error
 			err = nil
 		default:
-			// Other 400 Errors, 500 Errors even after retries. Log the error
-			log.Errorf("Error Calling CreateLogGroup:%v", err.Error())
+			if logGroupPresent, _ := service.IsLogGroupPresent(logGroup); logGroupPresent {
+				log.Debugf("skipping error check as the log group /%v/ already exists", logGroup)
+				err = nil
+			} else {
+				// Other 400 Errors, 500 Errors even after retries. Log the error
+				log.Errorf("Error Calling CreateLogGroup:%v", err.Error())
+				// Handle the common AWS errors and update the stop policy accordingly
+				sdkutil.HandleAwsError(log, err, service.stopPolicy)
+			}
 		}
 	}
 	return
@@ -225,21 +230,27 @@ func (service *CloudWatchLogsService) CreateLogStream(logGroup, logStream string
 
 	//Calling the API
 	if _, err = service.cloudWatchLogsClient.CreateLogStream(params); err != nil {
-		// Handle the common AWS errors and update the stop policy accordingly
-		sdkutil.HandleAwsError(log, err, service.stopPolicy)
-
 		// Cast err to awserr.Error to get the Code
 		errorCode := sdkutil.GetAwsErrorCode(err)
 
 		switch errorCode {
 		// Check for error code. Note that the AWS Retrier has already made retries for the 5xx Response Codes
 		case resourceAlreadyExistsException:
+			log.Debugf("log stream /%v/ already exists in the log group /%v/", logStream, logGroup)
 			// 400 Error, occurs when the LogStream already exists
 			// Ignoring the error
 			err = nil
 		default:
-			// Other 400 Errors, 500 Errors even after retries. Log the error
-			log.Errorf("Error Calling CreateLogStream:%v", err.Error())
+			if service.IsLogStreamPresent(logGroup, logStream) {
+				log.Debugf("skipping error check as the log stream /%v/ already exists in the log group /%v/", logStream, logGroup)
+				err = nil
+			} else {
+				// Other 400 Errors, 500 Errors even after retries. Log the error
+				log.Errorf("Error Calling CreateLogStream:%v", err.Error())
+
+				// Handle the common AWS errors and update the stop policy accordingly
+				sdkutil.HandleAwsError(log, err, service.stopPolicy)
+			}
 		}
 	}
 	return
@@ -314,7 +325,7 @@ func (service *CloudWatchLogsService) DescribeLogStreams(logGroup, logStreamPref
 
 }
 
-//getLogGroupDetails Calls the DescribeLogGroups API to get the details of the loggroup specified. Returns nil if not found
+// getLogGroupDetails Calls the DescribeLogGroups API to get the details of the loggroup specified. Returns nil if not found
 func (service *CloudWatchLogsService) getLogGroupDetails(logGroup string) (logGroupDetails *cloudwatchlogs.LogGroup, err error) {
 	log := service.context.Log()
 	// Keeping the nextToken as empty in the beginning. Might get filled from response for subsequent calls
@@ -359,6 +370,11 @@ func (service *CloudWatchLogsService) IsLogGroupPresent(logGroup string) (bool,
 	return logGroupDetails != nil, logGroupDetails
 }
 
+// IsLogStreamPresent checks and returns true when the log stream is present
+func (service *CloudWatchLogsService) IsLogStreamPresent(logGroupName, logStreamName string) bool {
+	return service.getLogStreamDetails(logGroupName, logStreamName) != nil
+}
+
 // GetSequenceTokenForStream returns the current sequence token for the stream specified
 func (service *CloudWatchLogsService) GetSequenceTokenForStream(logGroupName, logStreamName string) (sequenceToken *string) {
 	logStream := service.getLogStreamDetails(logGroupName, logStreamName)
@@ -469,7 +485,7 @@ func (service *CloudWatchLogsService) retryPutWithNewSequenceToken(messages []*c
 	return service.PutLogEvents(messages, logGroupName, logStreamName, sequenceToken)
 }
 
-//IsLogGroupEncryptedWithKMS return true if the log group is encrypted with KMS key.
+// IsLogGroupEncryptedWithKMS return true if the log group is encrypted with KMS key.
 func (service *CloudWatchLogsService) IsLogGroupEncryptedWithKMS(logGroup *cloudwatchlogs.LogGroup) (bool, error) {
 	if logGroup == nil {
 		return false, nil
@@ -483,7 +499,7 @@ func (service *CloudWatchLogsService) IsLogGroupEncryptedWithKMS(logGroup *cloud
 	return false, nil
 }
 
-//StreamData streams data from the absoluteFilePath file to cloudwatch logs.
+// StreamData streams data from the absoluteFilePath file to cloudwatch logs.
 func (service *CloudWatchLogsService) StreamData(
 	logGroupName string,
 	logStreamName string,
@@ -577,7 +593,7 @@ func (service *CloudWatchLogsService) StreamData(
 		if err == nil {
 			// Set the last known line to current since the upload was successful.
 			lastKnownLineUploadedToCWL = currentLineNumber
-			log.Trace("Successfully uploaded message line %d to CloudWatch", currentLineNumber)
+			log.Tracef("Successfully uploaded message line %v to CloudWatch", currentLineNumber)
 		} else {
 			if errCode := sdkutil.GetAwsErrorCode(err); errCode == resourceNotFoundException {
 				// Log group or log stream not found due to resource change outside of client. Stop log streaming for session
@@ -596,7 +612,7 @@ func (service *CloudWatchLogsService) StreamData(
 	return success
 }
 
-//getNextMessage gets the next message to be uploaded to cloudwatch.
+// getNextMessage gets the next message to be uploaded to cloudwatch.
 func (service *CloudWatchLogsService) getNextMessage(
 	absoluteFilePath string,
 	lastKnownLineUploadedToCWL *int64,
@@ -612,8 +628,15 @@ func (service *CloudWatchLogsService) getNextMessage(
 	}
 	defer file.Close()
 
-	// Initialize reader.
-	reader := bufio.NewReaderSize(file, MessageLengthThresholdInBytes)
+	var messageThreshold int
+
+	if structuredLogs {
+		messageThreshold = StreamMessageLengthThresholdInBytes
+	} else {
+		messageThreshold = MessageLengthThresholdInBytes
+	}
+
+	reader := bufio.NewReaderSize(file, messageThreshold)
 
 	// Skip to the last uploaded line.
 	if *lastKnownLineUploadedToCWL > 0 {
@@ -626,7 +649,7 @@ func (service *CloudWatchLogsService) getNextMessage(
 			}
 			_, err = reader.ReadSlice(NewLineCharacter)
 		}
-		if err != nil && err != io.EOF {
+		if err != nil && err != io.EOF && err != bufio.ErrBufferFull {
 			log.Warnf("Error skipping to last uploaded Cloudwatch line: %v", err)
 			return
 		}
@@ -642,17 +665,15 @@ func (service *CloudWatchLogsService) getNextMessage(
 				break
 			}
 		}
-
 		// Process message if needed before uploading to CW
 		line = processMessage(log, line, cleanupControlCharacters)
-
 		// Check if message length threshold for the event has reached.
 		// If true, then construct event with existing message so that new line will get added to the next event.
 		// If false, then continue to append new line to existing message.
-		if (len(message) + len(line)) > MessageLengthThresholdInBytes {
+		if (len(message) + len(line)) > messageThreshold {
 			log.Tracef("Appending line to current Cloudwatch event message"+
-				" exceeds length limit %v bytes. [Line: %v]",
-				MessageLengthThresholdInBytes, *currentLineNumber)
+				" exceeds length limit %v bytes. [Line: %v] [Length: %v]",
+				messageThreshold, *currentLineNumber, len(message)+len(line))
 
 			event := service.buildEventInfo(message, structuredLogs)
 
