@@ -18,13 +18,14 @@ package cloudwatchlogspublisher
 import (
 	"encoding/json"
 	"errors"
+	"fmt"
 	"os"
 	"strings"
 	"testing"
 	"time"
 
 	cloudwatchlogspublisher_mock "github.com/aws/amazon-ssm-agent/agent/agentlogstocloudwatch/cloudwatchlogspublisher/mock"
-	"github.com/aws/amazon-ssm-agent/agent/context"
+	"github.com/aws/amazon-ssm-agent/agent/mocks/context"
 	"github.com/aws/amazon-ssm-agent/agent/sdkutil"
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/awserr"
@@ -113,6 +114,51 @@ func TestCloudWatchLogsService_CreateLogGroup(t *testing.T) {
 
 }
 
+func TestCloudWatchLogsService_CreateLogGroup_AccessDenied_LogGroupExists(t *testing.T) {
+	cloudWatchMock := cloudwatchlogspublisher_mock.NewClientMockDefault(logMock)
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cloudWatchMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+	}
+	var logGroups []*cloudwatchlogs.LogGroup
+	logGroup := &cloudwatchlogs.LogGroup{LogGroupName: &logGroupName}
+	logGroups = append(logGroups, logGroup)
+	logGroupOutput := &cloudwatchlogs.DescribeLogGroupsOutput{
+		LogGroups: logGroups,
+	}
+	createLogGrpOutput := cloudwatchlogs.CreateLogGroupOutput{}
+	accessDeniedError := fmt.Errorf("access denied")
+
+	cloudWatchMock.On("CreateLogGroup", mock.AnythingOfType("*cloudwatchlogs.CreateLogGroupInput")).Return(&createLogGrpOutput, accessDeniedError)
+	cloudWatchMock.On("DescribeLogGroups", mock.AnythingOfType("*cloudwatchlogs.DescribeLogGroupsInput")).Return(logGroupOutput, nil)
+
+	err := service.CreateLogGroup(logGroupName)
+
+	assert.NoError(t, err, "expected create log group to be successful")
+}
+
+func TestCloudWatchLogsService_CreateLogGroup_AccessDenied_LogGroupNotExists(t *testing.T) {
+
+	cloudWatchMock := cloudwatchlogspublisher_mock.NewClientMockDefault(logMock)
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cloudWatchMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+	}
+	createLogGrpOutput := cloudwatchlogs.CreateLogGroupOutput{}
+	accessDeniedError := fmt.Errorf("access denied")
+	noLogGroupError := fmt.Errorf("no log group error")
+	describeLogGroupOutput := &cloudwatchlogs.DescribeLogGroupsOutput{}
+
+	cloudWatchMock.On("CreateLogGroup", mock.AnythingOfType("*cloudwatchlogs.CreateLogGroupInput")).Return(&createLogGrpOutput, accessDeniedError)
+	cloudWatchMock.On("DescribeLogGroups", mock.AnythingOfType("*cloudwatchlogs.DescribeLogGroupsInput")).Return(describeLogGroupOutput, noLogGroupError)
+
+	err := service.CreateLogGroup(logGroupName)
+
+	assert.Error(t, err, "error expected while creating log group")
+}
+
 func TestCloudWatchLogsService_DescribeLogStreams(t *testing.T) {
 	service := CloudWatchLogsService{
 		context:              contextMock,
@@ -145,6 +191,51 @@ func TestCloudWatchLogsService_CreateLogStream(t *testing.T) {
 
 }
 
+func TestCloudWatchLogsService_CreateLogStream_AccessDenied_Exists(t *testing.T) {
+
+	cloudWatchMock := cloudwatchlogspublisher_mock.NewClientMockDefault(logMock)
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cloudWatchMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+	}
+	var logStreams []*cloudwatchlogs.LogStream
+	logStream := &cloudwatchlogs.LogStream{LogStreamName: &logStreamName}
+	logStreams = append(logStreams, logStream)
+	describeLogStreamOutput := &cloudwatchlogs.DescribeLogStreamsOutput{
+		LogStreams: logStreams,
+	}
+	createLogStreamOutput := cloudwatchlogs.CreateLogStreamOutput{}
+
+	cloudWatchMock.On("CreateLogStream", mock.AnythingOfType("*cloudwatchlogs.CreateLogStreamInput")).Return(&createLogStreamOutput, nil)
+	cloudWatchMock.On("DescribeLogStreams", mock.Anything).Return(describeLogStreamOutput, nil)
+
+	err := service.CreateLogStream("LogGroup", "LogStream")
+
+	assert.NoError(t, err, "CreateLogStream should be called successfully")
+}
+
+func TestCloudWatchLogsService_CreateLogStream_AccessDenied_NotExists(t *testing.T) {
+
+	cloudWatchMock := cloudwatchlogspublisher_mock.NewClientMockDefault(logMock)
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cloudWatchMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+	}
+	accessDeniedError := fmt.Errorf("access denied")
+	noLogStreamError := fmt.Errorf("no log stream error")
+	createLogStreamOutput := cloudwatchlogs.CreateLogStreamOutput{}
+	describeLogStreamOutput := &cloudwatchlogs.DescribeLogStreamsOutput{}
+
+	cloudWatchMock.On("CreateLogStream", mock.AnythingOfType("*cloudwatchlogs.CreateLogStreamInput")).Return(&createLogStreamOutput, accessDeniedError)
+	cloudWatchMock.On("DescribeLogStreams", mock.Anything).Return(describeLogStreamOutput, noLogStreamError)
+
+	err := service.CreateLogStream("LogGroup", logStreamName)
+
+	assert.Error(t, err, "error expected during CreateLogStream")
+}
+
 func TestCloudWatchLogsService_PutLogEvents(t *testing.T) {
 	service := CloudWatchLogsService{
 		context:              contextMock,
@@ -281,11 +372,8 @@ func TestCloudWatchLogsService_getNextMessage_largeline(t *testing.T) {
 	file, err := os.Create(fileName)
 	assert.Nil(t, err, "Failed to create test file")
 	// writing more than 200K bytes
-	var largeLine string
-	for i := 0; i < 50; i++ {
-		largeLine += strings.Join(input, " ")
-	}
-	file.Write([]byte(largeLine))
+	message := make([]byte, MessageLengthThresholdInBytes+10000, MessageLengthThresholdInBytes+10000)
+	file.Write(message)
 	file.Close()
 
 	// Deleting file
@@ -298,13 +386,41 @@ func TestCloudWatchLogsService_getNextMessage_largeline(t *testing.T) {
 	var actualLastKnownLineUploadedToCWL int64 = 0
 	var actualCurrentLineNumber int64 = 0
 	events, eof := service.getNextMessage(fileName, &actualLastKnownLineUploadedToCWL, &actualCurrentLineNumber, false, false)
-
 	// Compare results
 	// Since length of line exceeds max limit of 200K, messages will be sent in 2 separate events
 	assert.Equal(t, 2, len(events))
 	assert.False(t, eof)
-	assert.Equal(t, largeLine[:200000], *events[0].Message)
-	assert.Equal(t, largeLine[200000:], *events[1].Message)
+	assert.Equal(t, string(message[:200000]), *events[0].Message)
+	assert.Equal(t, string(message[200000:]), *events[1].Message)
+}
+
+func TestCloudWatchLogsService_buildEventInfo_stream_threshold_should_match(t *testing.T) {
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cwLogsClientMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+		isFileComplete:       true,
+	}
+
+	message := make([]byte, StreamMessageLengthThresholdInBytes, StreamMessageLengthThresholdInBytes)
+	event := service.buildEventInfo(message, true)
+	// Result event size should be less than cloudwatch event quota
+	eventBytes, err := json.Marshal(event)
+	assert.Nil(t, err)
+	assert.Less(t, len(eventBytes)+64, 262144)
+}
+
+func TestCloudWatchLogsService_buildEventInfo_threshold_should_match(t *testing.T) {
+	service := CloudWatchLogsService{
+		context:              contextMock,
+		cloudWatchLogsClient: cwLogsClientMock,
+		stopPolicy:           sdkutil.NewStopPolicy("Test", 0),
+		isFileComplete:       true,
+	}
+
+	message := make([]byte, MessageLengthThresholdInBytes, MessageLengthThresholdInBytes)
+	event := service.buildEventInfo(message, false)
+	assert.Less(t, len([]byte(*event.Message))+64, 262144)
 }
 
 func TestCloudWatchLogsService_getNextMessage_ending_with_newlinecharacter(t *testing.T) {
@@ -663,6 +779,7 @@ func TestCloudWatchLogsService_StreamData_MissingStreamPermissions(t *testing.T)
 	logGroups = append(logGroups, logGroup)
 
 	cwLogsClientMock.On("CreateLogStream", mock.Anything).Return(&cloudwatchlogs.CreateLogStreamOutput{}, errors.New("error"))
+	cwLogsClientMock.On("DescribeLogStreams", mock.Anything).Return(&cloudwatchlogs.DescribeLogStreamsOutput{}, nil)
 
 	go func() {
 		time.Sleep(1800 * time.Millisecond)
