@@ -295,9 +295,9 @@ func ExecuteCommand(
 	// configure environment variables
 	prepareEnvironment(context, command, envVars)
 
-	log.Debug()
 	log.Debugf("Running in directory %v, command: %v %v", workingDir, commandName, commandArguments)
-	log.Debug()
+
+	quiesce()
 	if err = command.Start(); err != nil {
 		log.Error("error occurred starting the command", err)
 		exitCode = 1
@@ -412,9 +412,9 @@ func StartCommand(context context.T,
 	// configure environment variables
 	prepareEnvironment(context, command, make(map[string]string))
 
-	log.Debug()
 	log.Debugf("Running in directory %v, command: %v %v", workingDir, commandName, commandArguments)
-	log.Debug()
+
+	quiesce()
 	if err = command.Start(); err != nil {
 		log.Error("error occurred starting the command: ", err)
 		exitCode = 1
