@@ -11,6 +11,7 @@
 // either express or implied. See the License for the specific language governing
 // permissions and limitations under the License.
 
+//go:build windows
 // +build windows
 
 package executers
@@ -28,6 +29,10 @@ func prepareProcess(command *exec.Cmd) {
 	// nothing to do on windows
 }
 
+func quiesce() {
+	// not needed for Darwin workaround
+}
+
 func killProcess(process *os.Process, signal *timeoutSignal) error {
 	// process kill doesn't send proper signal to the process status
 	// Setting the signal to indicate execution was interrupted
