@@ -157,9 +157,9 @@ func validateAndReplaceSessionDocumentParameters(context context.T, params map[s
 		}
 	}
 
-	log.Info("Validating SSM parameters")
+	log.Debug("Validating SSM parameters")
 	// Validates SSM parameters
-	if err := parameterstore.ValidateSSMParameters(context, docContent.Parameters, validParameters); err != nil {
+	if err := parameterstore.ValidateSSMParameters(context, docContent.Parameters, validParameters, ""); err != nil {
 		return err
 	}
 
@@ -466,9 +466,9 @@ func getValidatedParameters(context context.T, params map[string]interface{}, do
 		}
 	}
 
-	log.Info("Validating SSM parameters")
+	log.Debug("Validating SSM parameters")
 	// Validates SSM parameters
-	if err := parameterstore.ValidateSSMParameters(context, docContent.Parameters, validParameters); err != nil {
+	if err := parameterstore.ValidateSSMParameters(context, docContent.Parameters, validParameters, docContent.InvokedPlugin); err != nil {
 		return err
 	}
 
