@@ -11,6 +11,7 @@
 // either express or implied. See the License for the specific language governing
 // permissions and limitations under the License.
 //
+//go:build windows
 // +build windows
 
 package configurecontainers
@@ -26,5 +27,5 @@ func runInstallCommands(context context.T, pluginInput ConfigureContainerPluginI
 }
 
 func runUninstallCommands(context context.T, pluginInput ConfigureContainerPluginInput, orchestrationDirectory string, out iohandler.IOHandler) {
-	windowscontainerutil.RunUninstallCommands(context.Log(), orchestrationDirectory, out)
+	windowscontainerutil.RunUninstallCommands(context, orchestrationDirectory, out)
 }
