@@ -17,6 +17,7 @@ import (
 	"testing"
 
 	"github.com/aws/amazon-ssm-agent/agent/context"
+	contextmocks "github.com/aws/amazon-ssm-agent/agent/mocks/context"
 	"github.com/aws/amazon-ssm-agent/agent/plugins/inventory/model"
 	"github.com/stretchr/testify/assert"
 )
@@ -35,7 +36,7 @@ func testCollectBillingInfoData(context context.T) (data []model.BillingInfoData
 }
 
 func TestGatherer(t *testing.T) {
-	contextMock := context.NewMockDefault()
+	contextMock := contextmocks.NewMockDefault()
 	gatherer := Gatherer(contextMock)
 	collectData = testCollectBillingInfoData
 	item, err := gatherer.Run(contextMock, model.Config{})
