@@ -4,7 +4,6 @@ import (
 	"time"
 
 	"github.com/aws/amazon-ssm-agent/agent/context"
-	"github.com/aws/amazon-ssm-agent/agent/contracts"
 	"github.com/aws/amazon-ssm-agent/agent/plugins/inventory/model"
 )
 
@@ -52,7 +51,6 @@ func (t *T) Run(context context.T, configuration model.Config) (items []model.It
 }
 
 // RequestStop stops the execution of application gatherer.
-func (t *T) RequestStop(stopType contracts.StopType) error {
-	var err error
-	return err
+func (t *T) RequestStop() error {
+	return nil
 }
