@@ -15,11 +15,6 @@ package instancedetailedinformation
 
 import "fmt"
 
-func MockTestExecutorWithError(command string, args ...string) ([]byte, error) {
-	var result []byte
-	return result, fmt.Errorf("Random Error")
-}
-
 // createMockExecutor creates an executor that returns the given stdout values on subsequent invocations.
 // If the number of invocations exceeds the number of outputs provided, the executor will return the last output.
 // For example createMockExecutor("a", "b", "c") will return an executor that returns the following values:
@@ -36,3 +31,24 @@ func createMockExecutor(stdout ...string) func(string, ...string) ([]byte, error
 		return []byte(stdout[index-1]), nil
 	}
 }
+
+// createMockExecutorWithErrorOnNthExecution creates an executor that returns the given stdout values on subsequent invocations and returns
+// an error on the Nth execution.
+// If the number of invocations exceeds the number of outputs provided, the executor will return the last output.
+// For example createMockExecutorWithErrorOnNthExecution(2, "a", "b", "c") will return an executor that returns the following values:
+// on first call -> "a"
+// on second call -> Error
+// on third call -> "c"
+// on every call after that -> "c"
+func createMockExecutorWithErrorOnNthExecution(n int, stdout ...string) func(string, ...string) ([]byte, error) {
+	var index = 0
+	return func(string, ...string) ([]byte, error) {
+		if n == index+1 || stdout == nil {
+			return []byte{}, fmt.Errorf("Random Error")
+		}
+		if index < len(stdout) {
+			index += 1
+		}
+		return []byte(stdout[index-1]), nil
+	}
+}
