@@ -11,6 +11,7 @@
 // either express or implied. See the License for the specific language governing
 // permissions and limitations under the License.
 
+//go:build windows
 // +build windows
 
 // Package updateec2config implements the UpdateEC2Config plugin.
@@ -22,6 +23,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"regexp"
 	"strconv"
 	"strings"
 	"time"
@@ -40,6 +42,12 @@ import (
 	"github.com/aws/amazon-ssm-agent/agent/updateutil"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateconstants"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo"
+	"github.com/aws/amazon-ssm-agent/agent/versionutil"
+)
+
+var (
+	ec2ConfigVersionPattern = regexp.MustCompile("[.0-9]+")
+	sourcePattern           = regexp.MustCompile(`^https?:[\/][\/][\w.-]+:?[\/\w.-]+$`)
 )
 
 // Plugin is the type for the RunCommand plugin.
@@ -64,9 +72,11 @@ type UpdatePluginConfig struct {
 	ManifestLocation string
 }
 
-type updateManager struct{}
+type updateManager struct {
+	context context.T
+}
 
-//TODO move the interface and structs into a separate file to reduce the size of this main file
+// TODO move the interface and structs into a separate file to reduce the size of this main file
 // pluginHelper is a interface that has helper functions for update manager
 type pluginHelper interface {
 	generateSetupUpdateCmd(log log.T,
@@ -130,9 +140,10 @@ func getEC2ConfigCurrentVersion(log log.T) (res string, err error) {
 	}
 
 	data := string(cmdOut)
-	if len(data) > 1 {
-		version := strings.TrimSpace(data)
-		log.Debug("GetEC2ConfigCurrentVersion: version after trimming space is ", version)
+	matches := ec2ConfigVersionPattern.FindStringSubmatch(data)
+	if matches != nil {
+		version := matches[0]
+		log.Debugf("GetEC2ConfigCurrentVersion: version after trimming space is %s", version)
 		return version, err
 	}
 
@@ -182,6 +193,14 @@ func runUpdateAgent(
 	}
 	//Calculate manifest location base on current instance's region
 	pluginInput.Source = strings.Replace(pluginInput.Source, updateconstants.RegionHolder, region, -1)
+
+	//Check the format of the source input and revert to default if not
+	if err = validateSource(pluginInput); err != nil {
+		log.Warnf("Invalid url provided at %v (url must be https/http link) falling back to default manifest location", pluginInput.Source)
+		pluginInput.Source = p.ManifestLocation
+		pluginInput.Source = strings.Replace(pluginInput.Source, updateconstants.RegionHolder, region, -1)
+	}
+
 	//Calculate updater package name base on agent name
 	pluginInput.UpdaterName = pluginInput.AgentName + updateconstants.UpdaterPackageNamePrefix
 	//Generate update output
@@ -271,7 +290,16 @@ func runUpdateAgent(
 			appconfig.EC2UpdateArtifactsRoot, pluginInput.UpdaterName, updaterVersion)
 
 		//Command to setup the installation
-		if _, _, err = util.ExeCommand(log, cmd, workDir, appconfig.EC2UpdateArtifactsRoot, pluginConfig.StdoutFileName, pluginConfig.StderrFileName, false); err != nil {
+		commandInput := &updateutil.CommandExecutionSettings{
+			Log:         log,
+			Cmd:         strings.Fields(cmd),
+			WorkingDir:  workDir,
+			UpdaterRoot: appconfig.EC2UpdateArtifactsRoot,
+			StdOut:      pluginConfig.StdoutFileName,
+			StdErr:      pluginConfig.StderrFileName,
+			IsAsync:     false,
+		}
+		if _, _, err = util.ExeCommand(commandInput); err != nil {
 			output.MarkAsFailed(err)
 			return
 		}
@@ -286,7 +314,16 @@ func runUpdateAgent(
 			return
 		}
 		log.Debugf("Setup update command %v", cmd)
-		if _, _, err = util.ExeCommand(log, cmd, workDir, appconfig.EC2UpdateArtifactsRoot, pluginConfig.StdoutFileName, pluginConfig.StderrFileName, true); err != nil {
+		asyncCommandInput := &updateutil.CommandExecutionSettings{
+			Log:         log,
+			Cmd:         strings.Fields(cmd),
+			WorkingDir:  workDir,
+			UpdaterRoot: appconfig.EC2UpdateArtifactsRoot,
+			StdOut:      pluginConfig.StdoutFileName,
+			StdErr:      pluginConfig.StderrFileName,
+			IsAsync:     true,
+		}
+		if _, _, err = util.ExeCommand(asyncCommandInput); err != nil {
 			output.MarkAsFailed(err)
 			return
 		}
@@ -336,12 +373,7 @@ func (m *updateManager) generateSetupUpdateCmd(log log.T,
 
 	cmd = updateutil.BuildUpdateCommand(cmd, HistoryCmd, numHistories)
 
-	var appConfig appconfig.SsmagentConfig
-	appConfig, err = appconfig.Config(false)
-	if err != nil {
-		log.Error("something went wrong while generating the setup installation command")
-		return "", err
-	}
+	appConfig := m.context.AppConfig()
 
 	cmd = updateutil.BuildUpdateCommand(cmd, MdsEndpointCmd, appConfig.Mds.Endpoint)
 
@@ -365,12 +397,7 @@ func (m *updateManager) generateUpdateCmd(log log.T,
 
 	cmd = updateutil.BuildUpdateCommand(cmd, HistoryCmd, numHistories)
 
-	var appConfig appconfig.SsmagentConfig
-	appConfig, err = appconfig.Config(false)
-	if err != nil {
-		log.Error("something went wrong while generating update command")
-		return "", err
-	}
+	appConfig := m.context.AppConfig()
 
 	cmd = updateutil.BuildUpdateCommand(cmd, MdsEndpointCmd, appConfig.Mds.Endpoint)
 
@@ -495,14 +522,19 @@ func (m *updateManager) validateUpdate(log log.T,
 		return true, err
 	}
 
-	if pluginInput.TargetVersion == currentVersion {
+	var versionCmpVal int
+	if versionCmpVal, err = versionutil.VersionCompare(pluginInput.TargetVersion, currentVersion); err != nil {
+		return true, err
+	}
+
+	if versionCmpVal == 0 {
 		out.AppendInfof("%v %v has already been installed, update skipped",
 			pluginInput.AgentName,
 			currentVersion)
 		out.MarkAsSucceeded()
 		return true, nil
 	}
-	if pluginInput.TargetVersion < currentVersion && !allowDowngrade {
+	if versionCmpVal < 0 && !allowDowngrade {
 		return true,
 			fmt.Errorf(
 				"updating %v to an older version, please enable allow downgrade to proceed",
@@ -534,7 +566,9 @@ func (p *Plugin) Execute(config contracts.Configuration, cancelFlag task.CancelF
 	log.Info("RunCommand started with update configuration for EC2 config update ", config)
 	util := new(updateutil.Utility)
 	util.Context = p.context
-	manager := new(updateManager)
+	manager := &updateManager{
+		p.context,
+	}
 
 	if cancelFlag.ShutDown() {
 		output.MarkAsShutdown()
@@ -561,7 +595,7 @@ func Name() string {
 // GetUpdatePluginConfig returns the default values for the update plugin
 func GetUpdatePluginConfig(context context.T) UpdatePluginConfig {
 	var manifestURL string
-	dynamicS3Endpoint := context.Identity().GetDefaultEndpoint("s3")
+	dynamicS3Endpoint := context.Identity().GetServiceEndpoint("s3")
 	if dynamicS3Endpoint != "" {
 		manifestURL = "https://" + dynamicS3Endpoint + ManifestPath
 	} else {
@@ -577,3 +611,12 @@ func GetUpdatePluginConfig(context context.T) UpdatePluginConfig {
 		ManifestLocation: manifestURL,
 	}
 }
+
+func validateSource(pluginInput UpdatePluginInput) (err error) {
+	//This pattern allows for http/https links to local hosts and s3 buckets
+	validSourceValue := sourcePattern
+	if !validSourceValue.MatchString(pluginInput.Source) {
+		return errors.New("Invalid source")
+	}
+	return err
+}
