@@ -15,6 +15,7 @@
 package updatessmagent
 
 import (
+	"bytes"
 	"fmt"
 	"testing"
 	"time"
@@ -23,7 +24,9 @@ import (
 	"github.com/aws/amazon-ssm-agent/agent/contracts"
 	"github.com/aws/amazon-ssm-agent/agent/framework/processor/executer/iohandler"
 	"github.com/aws/amazon-ssm-agent/agent/log"
-	"github.com/aws/amazon-ssm-agent/agent/task"
+	contextmocks "github.com/aws/amazon-ssm-agent/agent/mocks/context"
+	lockmocks "github.com/aws/amazon-ssm-agent/agent/mocks/lockfile"
+	"github.com/aws/amazon-ssm-agent/agent/mocks/task"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateconstants"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo"
@@ -38,23 +41,24 @@ import (
 	"github.com/stretchr/testify/mock"
 )
 
-var mockContext = context.NewMockDefault()
+var mockContext = contextmocks.NewMockDefault()
 
 func TestGenerateUpdateCmd(t *testing.T) {
 	pluginInput := createStubPluginInput()
 
 	result, err := generateUpdateCmd(pluginInput,
-		"3.0.0.0", "messageID", "stdout", "stderr", "prefix", "bucket")
+		"3.0.0.0", "messageID with space", contracts.MessageGatewayService, "stdout", "stderr", "prefix", "bucket")
 
 	assert.NoError(t, err)
-	assert.Contains(t, result, "3.0.0.0")
-	assert.Contains(t, result, "messageID")
-	assert.Contains(t, result, "stdout")
-	assert.Contains(t, result, "stderr")
-	assert.Contains(t, result, "prefix")
-	assert.Contains(t, result, "bucket")
-	assert.Contains(t, result, "manifest")
-	assert.NotContains(t, result, "-"+updateconstants.DisableDowngradeCmd)
+	assert.EqualValues(t, 22, len(result))
+	assert.Contains(t, result[0], "3.0.0.0")
+	assert.EqualValues(t, "messageID with space", result[9])
+	assert.EqualValues(t, "stdout", result[11])
+	assert.EqualValues(t, "stderr", result[13])
+	assert.EqualValues(t, "prefix", result[15])
+	assert.EqualValues(t, "bucket", result[17])
+	assert.EqualValues(t, "testSource", result[19])
+	assert.EqualValues(t, "MessageGatewayService", result[21])
 }
 
 func TestGenerateUpdateCmdNoDowngrade(t *testing.T) {
@@ -62,17 +66,18 @@ func TestGenerateUpdateCmdNoDowngrade(t *testing.T) {
 	pluginInput.AllowDowngrade = "false"
 
 	result, err := generateUpdateCmd(pluginInput,
-		"3.0.0.0", "messageID", "stdout", "stderr", "prefix", "bucket")
-
+		"3.0.0.0", "messageID with space", contracts.MessageGatewayService, "stdout", "stderr", "prefix", "bucket")
 	assert.NoError(t, err)
-	assert.Contains(t, result, "3.0.0.0")
-	assert.Contains(t, result, "messageID")
-	assert.Contains(t, result, "stdout")
-	assert.Contains(t, result, "stderr")
-	assert.Contains(t, result, "prefix")
-	assert.Contains(t, result, "bucket")
-	assert.Contains(t, result, "manifest")
-	assert.Contains(t, result, "-"+updateconstants.DisableDowngradeCmd)
+	assert.EqualValues(t, 23, len(result))
+	assert.Contains(t, result[0], "3.0.0.0")
+	assert.EqualValues(t, "messageID with space", result[10])
+	assert.EqualValues(t, "stdout", result[12])
+	assert.EqualValues(t, "stderr", result[14])
+	assert.EqualValues(t, "prefix", result[16])
+	assert.EqualValues(t, "bucket", result[18])
+	assert.EqualValues(t, "testSource", result[20])
+	assert.EqualValues(t, "MessageGatewayService", result[22])
+	assert.EqualValues(t, result[2], "-"+updateconstants.DisableDowngradeCmd)
 }
 
 func TestGenerateUpdateCmdInvalidDowngrade(t *testing.T) {
@@ -80,7 +85,7 @@ func TestGenerateUpdateCmdInvalidDowngrade(t *testing.T) {
 	pluginInput.AllowDowngrade = "somerandomstring"
 
 	_, err := generateUpdateCmd(pluginInput,
-		"3.0.0.0", "messageID", "stdout", "stderr", "prefix", "bucket")
+		"3.0.0.0", "messageID", contracts.MessageGatewayService, "stdout", "stderr", "prefix", "bucket")
 
 	assert.Error(t, err)
 }
@@ -110,7 +115,10 @@ func TestUpdateAgent_FailedDownloadManifest(t *testing.T) {
 	manifest := &updatemanifestmocks.T{}
 
 	s3Util := &updates3utilmocks.T{}
-	s3Util.On("DownloadManifest", mock.Anything, pluginInput.Source).Return(fmt.Errorf("SomeDownloadManifestError"))
+	s3Util.On("DownloadManifest", mock.Anything, pluginInput.Source).Return(
+		&updates3util.UpdateErrorStruct{
+			Error: fmt.Errorf("SomeDownloadManifestError"),
+		})
 
 	out := iohandler.DefaultIOHandler{}
 	execMock := &executormocks.IExecutor{}
@@ -355,12 +363,12 @@ func TestExecute(t *testing.T) {
 	p[0] = pluginInput
 	config.Properties = p
 	plugin := &Plugin{
-		Context: context.NewMockDefault(),
+		Context: contextmocks.NewMockDefault(),
 	}
 
 	pluginInput.TargetVersion = ""
 	mockCancelFlag := new(task.MockCancelFlag)
-	mockLockfile := lockfile.MockLockfile{}
+	mockLockfile := lockmocks.MockLockfile{}
 	mockIOHandler := iohandler.DefaultIOHandler{}
 	methodCalled := false
 
@@ -382,7 +390,7 @@ func TestExecute(t *testing.T) {
 	}
 
 	getLockObj = func(pth string) (lockfile.Lockfile, error) {
-		return mockLockfile, nil
+		return &mockLockfile, nil
 	}
 	// Setup mocks
 	mockCancelFlag.On("Canceled").Return(false)
@@ -412,15 +420,15 @@ func TestExecuteUpdateLocked(t *testing.T) {
 	p[0] = pluginInput
 	config.Properties = p
 	plugin := &Plugin{
-		Context: context.NewMockDefault(),
+		Context: contextmocks.NewMockDefault(),
 	}
 	pluginInput.TargetVersion = ""
 	mockCancelFlag := new(task.MockCancelFlag)
-	mockLockfile := lockfile.MockLockfile{}
+	mockLockfile := lockmocks.MockLockfile{}
 	mockIOHandler := iohandler.DefaultIOHandler{}
 
 	getLockObj = func(pth string) (lockfile.Lockfile, error) {
-		return mockLockfile, nil
+		return &mockLockfile, nil
 	}
 
 	// Setup mocks
@@ -444,11 +452,11 @@ func TestExecutePanicDuringUpdate(t *testing.T) {
 	p[0] = pluginInput
 	config.Properties = p
 	plugin := &Plugin{
-		Context: context.NewMockDefault(),
+		Context: contextmocks.NewMockDefault(),
 	}
 	pluginInput.TargetVersion = ""
 	mockCancelFlag := new(task.MockCancelFlag)
-	mockLockfile := lockfile.MockLockfile{}
+	mockLockfile := lockmocks.MockLockfile{}
 	mockIOHandler := iohandler.DefaultIOHandler{}
 	methodCalled := false
 
@@ -466,11 +474,10 @@ func TestExecutePanicDuringUpdate(t *testing.T) {
 		downloadFolder string) int {
 		methodCalled = true
 		panic(fmt.Errorf("Some Random Panic"))
-		return 1
 	}
 
 	getLockObj = func(pth string) (lockfile.Lockfile, error) {
-		return mockLockfile, nil
+		return &mockLockfile, nil
 	}
 
 	// Setup mocks
@@ -500,11 +507,11 @@ func TestExecuteFailureDuringUpdate(t *testing.T) {
 	p[0] = pluginInput
 	config.Properties = p
 	plugin := &Plugin{
-		Context: context.NewMockDefault(),
+		Context: contextmocks.NewMockDefault(),
 	}
 	pluginInput.TargetVersion = ""
 	mockCancelFlag := new(task.MockCancelFlag)
-	mockLockfile := lockfile.MockLockfile{}
+	mockLockfile := lockmocks.MockLockfile{}
 	mockIOHandler := iohandler.DefaultIOHandler{}
 	methodCalled := false
 
@@ -526,7 +533,7 @@ func TestExecuteFailureDuringUpdate(t *testing.T) {
 	}
 
 	getLockObj = func(pth string) (lockfile.Lockfile, error) {
-		return mockLockfile, nil
+		return &mockLockfile, nil
 	}
 	// Setup mocks
 	mockCancelFlag.On("Canceled").Return(false)
@@ -610,18 +617,28 @@ func (u *fakeUtility) CreateUpdateDownloadFolder() (folder string, err error) {
 	return "", nil
 }
 
-func (u *fakeUtility) ExeCommand(
-	log log.T,
-	cmd string,
-	updateRoot string,
-	workingDir string,
-	stdOut string,
-	stdErr string,
-	isAsync bool) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error) {
+func (u *fakeUtility) ExeCommand(commandInput *updateutil.CommandExecutionSettings) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error) {
 	u.retryCounter++
 	return u.pid, exitCode, u.execCommandError
 }
 
+func (u *fakeUtility) ExeCommandWithSlice(commandInput *updateutil.CommandExecutionSettings) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error) {
+	u.retryCounter++
+	return u.pid, exitCode, u.execCommandError
+}
+
+func (u *fakeUtility) ExecCommandWithOutput(commandInput *updateutil.CommandExecutionSettings) (pId int, exitCode updateconstants.UpdateScriptExitCode, stdoutBytes *bytes.Buffer, errorBytes *bytes.Buffer, cmdErr error) {
+	u.retryCounter++
+	return u.pid, exitCode, nil, nil, u.execCommandError
+}
+
+func (u *fakeUtility) UpdateExecutionTimeOut(int) {
+	return
+}
+func (u *fakeUtility) GetExecutionTimeOut() int {
+	return 0
+}
+
 func (u *fakeUtility) SaveUpdatePluginResult(
 	log log.T,
 	updateRoot string,
@@ -632,3 +649,15 @@ func (u *fakeUtility) SaveUpdatePluginResult(
 func (u *fakeUtility) IsDiskSpaceSufficientForUpdate(log log.T) (bool, error) {
 	return !u.noDiskSpace, u.isDiskSpaceErr
 }
+
+func (u *fakeUtility) LoadUpdateDocumentState(ctx context.T, commandId string) error {
+	return nil
+}
+
+func (u *fakeUtility) UpdateInstallDelayer(ctx context.T, updateRoot string) error {
+	return nil
+}
+
+func (u *fakeUtility) VerifyInstalledVersion(log log.T, targetVersion string) updateconstants.ErrorCode {
+	return ""
+}
