@@ -16,7 +16,7 @@ package s3util
 import (
 	"testing"
 
-	"github.com/aws/amazon-ssm-agent/agent/context"
+	"github.com/aws/amazon-ssm-agent/agent/mocks/context"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -28,11 +28,6 @@ type s3EndpointTest struct {
 var (
 	getFallbackS3EndpointTests = []s3EndpointTest{
 		// {region, output},
-		{"us-east-1", "s3.amazonaws.com"},
-		{"us-west-1", "s3.amazonaws.com"},
-		{"af-south-1", "s3.amazonaws.com"},
-		{"eu-south-1", "s3.amazonaws.com"},
-		{"ap-southeast-2", "s3.amazonaws.com"},
 		{"us-gov-east-1", "s3.us-gov-west-1.amazonaws.com"},
 		{"us-gov-west-1", "s3.us-gov-east-1.amazonaws.com"},
 		{"cn-north-1", "s3.cn-northwest-1.amazonaws.com.cn"},
@@ -46,12 +41,3 @@ func TestGetFallbackS3Endpoint(t *testing.T) {
 		assert.Equal(t, test.output, output, "The two urls should be the same")
 	}
 }
-
-func TestIsKnownRegion(t *testing.T) {
-	assert.True(t, isKnownRegion("us-east-1"))
-	assert.True(t, isKnownRegion("eu-west-1"))
-	assert.True(t, isKnownRegion("us-gov-east-1"))
-	assert.True(t, isKnownRegion("cn-north-1"))
-	assert.False(t, isKnownRegion(""))
-	assert.False(t, isKnownRegion("not-a-region"))
-}
