@@ -24,12 +24,12 @@ import (
 	"math"
 	"math/rand"
 	"runtime/debug"
+	"strings"
 	"sync"
 	"time"
 
 	"github.com/aws/amazon-ssm-agent/agent/context"
 	"github.com/aws/amazon-ssm-agent/agent/log"
-	"github.com/aws/amazon-ssm-agent/agent/rip"
 	"github.com/aws/amazon-ssm-agent/agent/session/communicator"
 	mgsConfig "github.com/aws/amazon-ssm-agent/agent/session/config"
 	mgsContracts "github.com/aws/amazon-ssm-agent/agent/session/contracts"
@@ -38,6 +38,7 @@ import (
 	"github.com/aws/amazon-ssm-agent/agent/session/service"
 	"github.com/aws/amazon-ssm-agent/agent/task"
 	"github.com/aws/amazon-ssm-agent/agent/version"
+	"github.com/aws/amazon-ssm-agent/agent/versionutil"
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/gorilla/websocket"
 	"github.com/twinj/uuid"
@@ -48,7 +49,9 @@ const (
 	sequenceNumber = 0
 	messageFlags   = 3
 	// Timeout period before a handshake operation expires on the agent.
-	handshakeTimeout = 15 * time.Second
+	handshakeTimeout                        = 15 * time.Second
+	clientVersionWithoutOutputSeparation    = "1.2.295"
+	firstVersionWithOutputSeparationFeature = "1.2.312.0"
 )
 
 type IDataChannel interface {
@@ -74,6 +77,8 @@ type IDataChannel interface {
 	GetRegion() string
 	IsActive() bool
 	PrepareToCloseChannel(log log.T)
+	GetSeparateOutputPayload() bool
+	SetSeparateOutputPayload(separateOutputPayload bool)
 }
 
 // DataChannel used for session communication between the message gateway service and the agent.
@@ -90,6 +95,9 @@ type DataChannel struct {
 	ExpectedSequenceNumber int64
 	//records sequence number of last stream data message sent over data channel
 	StreamDataSequenceNumber int64
+	//ensure only one goroutine sending message with current StreamDataSequenceNumber in data channel
+	//check carefully for deadlock when not reusing of SendStreamDataMessage
+	StreamDataSequenceNumberMutex *sync.Mutex
 	//buffer to store outgoing stream messages until acknowledged
 	//using linked list for this buffer as access to oldest message is required and it support faster deletion from any position of list
 	OutgoingMessageBuffer ListMessageBuffer
@@ -111,7 +119,8 @@ type DataChannel struct {
 	//blockCipher stores encrytion keys and provides interface for encryption/decryption functions
 	blockCipher crypto.IBlockCipher
 	// Indicates whether encryption was enabled
-	encryptionEnabled bool
+	encryptionEnabled     bool
+	separateOutputPayload bool
 }
 
 type ListMessageBuffer struct {
@@ -234,6 +243,7 @@ func (dataChannel *DataChannel) Initialize(context context.T,
 	dataChannel.Pause = false
 	dataChannel.ExpectedSequenceNumber = 0
 	dataChannel.StreamDataSequenceNumber = 0
+	dataChannel.StreamDataSequenceNumberMutex = &sync.Mutex{}
 	dataChannel.OutgoingMessageBuffer = ListMessageBuffer{
 		list.New(),
 		mgsConfig.OutgoingMessageBufferCapacity,
@@ -261,6 +271,44 @@ func (dataChannel *DataChannel) Initialize(context context.T,
 	}
 }
 
+func (dataChannel *DataChannel) getWsChannelOnErrorHandler(mgsService service.Service,
+	sessionId string,
+	clientId string,
+	log log.T) func(error) {
+
+	return func(err error) {
+		uuid.SwitchFormat(uuid.CleanHyphen)
+		requestId := uuid.NewV4().String()
+		callable := func() (channel interface{}, err error) {
+			tokenValue, err := getDataChannelToken(log, mgsService, sessionId, requestId, clientId)
+			if err != nil {
+				log.Debugf("getDataChannelToken received error: %v", err)
+				if strings.Contains(err.Error(), mgsConfig.SessionAlreadyTerminatedError) {
+					log.Info("Setting task to cancelled as session is already terminated")
+					dataChannel.cancelFlag.Set(task.Canceled)
+				}
+				return dataChannel, err
+			}
+			dataChannel.wsChannel.SetChannelToken(tokenValue)
+			if err = dataChannel.Reconnect(log); err != nil {
+				return dataChannel, err
+			}
+			return dataChannel, nil
+		}
+		retryer := retry.ExponentialRetryer{
+			CallableFunc:        callable,
+			GeometricRatio:      mgsConfig.RetryGeometricRatio,
+			InitialDelayInMilli: rand.Intn(mgsConfig.DataChannelRetryInitialDelayMillis) + mgsConfig.DataChannelRetryInitialDelayMillis,
+			MaxDelayInMilli:     mgsConfig.DataChannelRetryMaxIntervalMillis,
+			MaxAttempts:         mgsConfig.DataChannelNumMaxAttempts,
+			NonRetryableErrors:  getNonRetryableDataChannelErrors(),
+		}
+		if _, err := retryer.Call(); err != nil {
+			log.Error(err)
+		}
+	}
+}
+
 // SetWebSocket populates webchannel object.
 func (dataChannel *DataChannel) SetWebSocket(context context.T,
 	mgsService service.Service,
@@ -279,31 +327,7 @@ func (dataChannel *DataChannel) SetWebSocket(context context.T,
 		return err
 	}
 
-	onErrorHandler := func(err error) {
-		uuid.SwitchFormat(uuid.CleanHyphen)
-		requestId := uuid.NewV4().String()
-		callable := func() (channel interface{}, err error) {
-			tokenValue, err := getDataChannelToken(log, mgsService, sessionId, requestId, clientId)
-			if err != nil {
-				return dataChannel, err
-			}
-			dataChannel.wsChannel.SetChannelToken(tokenValue)
-			if err = dataChannel.Reconnect(log); err != nil {
-				return dataChannel, err
-			}
-			return dataChannel, nil
-		}
-		retryer := retry.ExponentialRetryer{
-			CallableFunc:        callable,
-			GeometricRatio:      mgsConfig.RetryGeometricRatio,
-			InitialDelayInMilli: rand.Intn(mgsConfig.DataChannelRetryInitialDelayMillis) + mgsConfig.DataChannelRetryInitialDelayMillis,
-			MaxDelayInMilli:     mgsConfig.DataChannelRetryMaxIntervalMillis,
-			MaxAttempts:         mgsConfig.DataChannelNumMaxAttempts,
-		}
-		if _, err := retryer.Call(); err != nil {
-			log.Error(err)
-		}
-	}
+	onErrorHandler := dataChannel.getWsChannelOnErrorHandler(mgsService, sessionId, clientId, log)
 
 	if err := dataChannel.wsChannel.Initialize(context,
 		sessionId,
@@ -317,13 +341,14 @@ func (dataChannel *DataChannel) SetWebSocket(context context.T,
 		log.Errorf("failed to initialize websocket channel for datachannel, error: %s", err)
 		return err
 	}
+
 	return nil
 }
 
 // Open opens the websocket connection and sends the token for service to acknowledge the connection.
 func (dataChannel *DataChannel) Open(log log.T) error {
 	// Opens websocket connection
-	if err := dataChannel.wsChannel.Open(log); err != nil {
+	if err := dataChannel.wsChannel.Open(log, nil); err != nil {
 		return fmt.Errorf("failed to connect data channel with error: %s", err)
 	}
 
@@ -399,18 +424,22 @@ func (dataChannel *DataChannel) SendStreamDataMessage(log log.T, payloadType mgs
 		return nil
 	}
 
-	var flag uint64 = 0
-	if dataChannel.StreamDataSequenceNumber == 0 {
-		flag = 1
-	}
-
 	// If encryption has been enabled, encrypt the payload
-	if dataChannel.encryptionEnabled && payloadType == mgsContracts.Output {
+	if dataChannel.encryptionEnabled && (payloadType == mgsContracts.Output || payloadType == mgsContracts.StdErr || payloadType == mgsContracts.ExitCode) {
 		if inputData, err = dataChannel.blockCipher.EncryptWithAESGCM(inputData); err != nil {
 			return fmt.Errorf("error encrypting stream data message sequence %d, err: %v", dataChannel.StreamDataSequenceNumber, err)
 		}
 	}
 
+	// StreamDataSequenceNumber only changed in the code block below
+	dataChannel.StreamDataSequenceNumberMutex.Lock()
+	defer dataChannel.StreamDataSequenceNumberMutex.Unlock()
+
+	var flag uint64 = 0
+	if dataChannel.StreamDataSequenceNumber == 0 {
+		flag = 1
+	}
+
 	uuid.SwitchFormat(uuid.CleanHyphen)
 	messageId := uuid.NewV4()
 	agentMessage := &mgsContracts.AgentMessage{
@@ -539,7 +568,7 @@ func (dataChannel *DataChannel) SendAgentSessionStateMessage(log log.T, sessionS
 		return err
 	}
 
-	log.Tracef("Send %s message with session status %s", mgsContracts.AgentSessionState, string(sessionStatus))
+	log.Debugf("Send %s message with session status %s", mgsContracts.AgentSessionState, string(sessionStatus))
 	if err := dataChannel.sendAgentMessage(log, mgsContracts.AgentSessionState, agentSessionStateContentBytes); err != nil {
 		return err
 	}
@@ -717,8 +746,11 @@ func (dataChannel *DataChannel) handleStreamDataMessage(log log.T,
 			dataChannel.AddDataToIncomingMessageBuffer(streamingMessage)
 		}
 	} else {
-		log.Tracef("Discarding already processed message. Received Sequence Number: %d. Expected Sequence Number: %d",
+		log.Debugf("Resending acknowledge for already processed message. Received Sequence Number: %d. Expected Sequence Number: %d",
 			streamDataMessage.SequenceNumber, dataChannel.ExpectedSequenceNumber)
+		if err = dataChannel.SendAcknowledgeMessage(log, streamDataMessage); err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -866,6 +898,7 @@ func (dataChannel *DataChannel) handleHandshakeResponse(log log.T, streamDataMes
 		}
 	}
 	dataChannel.handshake.clientVersion = handshakeResponse.ClientVersion
+	log.Infof("Client side session manager plugin version is: %s", handshakeResponse.ClientVersion)
 	dataChannel.handshake.responseChan <- true
 	return nil
 }
@@ -911,8 +944,12 @@ func (dataChannel *DataChannel) finalizeKMSEncryption(log log.T, actionResult js
 		return err
 	}
 
+	if dataChannel.context.AppConfig().Kms.RequireKMSChallengeResponse && !encryptionResponse.ChallengeAcknowledgement {
+		return fmt.Errorf("client does not support required encryption context random challenge")
+	}
+
 	sessionId := dataChannel.ChannelId // ChannelId is SessionId
-	if err := dataChannel.blockCipher.UpdateEncryptionKey(log, encryptionResponse.KMSCipherTextKey, sessionId, dataChannel.InstanceId); err != nil {
+	if err := dataChannel.blockCipher.UpdateEncryptionKey(log, encryptionResponse.KMSCipherTextKey, sessionId, dataChannel.InstanceId, encryptionResponse.ChallengeAcknowledgement); err != nil {
 		return fmt.Errorf("Fetching data key failed: %s", err)
 	}
 	dataChannel.encryptionEnabled = true
@@ -1005,7 +1042,8 @@ func (dataChannel *DataChannel) buildHandshakeRequestPayload(log log.T,
 			mgsContracts.RequestedClientAction{
 				ActionType: mgsContracts.KMSEncryption,
 				ActionParameters: mgsContracts.KMSEncryptionRequest{
-					KMSKeyID: dataChannel.blockCipher.GetKMSKeyId(),
+					KMSKeyID:  dataChannel.blockCipher.GetKMSKeyId(),
+					Challenge: dataChannel.blockCipher.GetRandomChallenge(),
 				}})
 	}
 
@@ -1018,9 +1056,17 @@ func (dataChannel *DataChannel) buildHandshakeCompletePayload(log log.T) mgsCont
 	handshakeComplete.HandshakeTimeToComplete =
 		dataChannel.handshake.handshakeEndTime.Sub(dataChannel.handshake.handshakeStartTime)
 
-	if dataChannel.encryptionEnabled == true {
-		handshakeComplete.CustomerMessage = "This session is encrypted using AWS KMS."
+	clientVersion := dataChannel.GetClientVersion()
+	if dataChannel.separateOutputPayload == true && versionutil.Compare(clientVersion, clientVersionWithoutOutputSeparation, true) <= 0 {
+		handshakeComplete.CustomerMessage = "Please update session manager plugin version (minimum required version " +
+			firstVersionWithOutputSeparationFeature +
+			") for fully support of separate StdOut/StdErr output.\r\n"
 	}
+
+	if dataChannel.encryptionEnabled == true {
+		handshakeComplete.CustomerMessage += "This session is encrypted using AWS KMS."
+	}
+
 	return handshakeComplete
 }
 
@@ -1109,6 +1155,17 @@ func (dataChannel *DataChannel) IsActive() bool {
 	return !dataChannel.Pause
 }
 
+// GetSeparateOutputPayload returns boolean value indicating separate
+// stdout/stderr output for non-interactive session or not
+func (dataChannel *DataChannel) GetSeparateOutputPayload() bool {
+	return dataChannel.separateOutputPayload
+}
+
+// SetSeparateOutputPayload set separateOutputPayload value
+func (dataChannel *DataChannel) SetSeparateOutputPayload(separateOutputPayload bool) {
+	dataChannel.separateOutputPayload = separateOutputPayload
+}
+
 // getDataChannelToken calls CreateDataChannel to get the token for this session.
 func getDataChannelToken(log log.T,
 	mgsService service.Service,
@@ -1127,13 +1184,13 @@ func getDataChannelToken(log log.T,
 		return "", fmt.Errorf("CreateDataChannel failed with no output or error: %s", err)
 	}
 
-	log.Debugf("Successfully get datachannel token")
+	log.Debug("Successfully get datachannel token")
 	return *createDataChannelOutput.TokenValue, nil
 }
 
 // getMgsEndpoint builds mgs endpoint.
 func getMgsEndpoint(context context.T, region string) (string, error) {
-	hostName := rip.GetMgsEndpoint(context, region)
+	hostName := mgsConfig.GetMgsEndpoint(context, region)
 	if hostName == "" {
 		return "", fmt.Errorf("no MGS endpoint found in region %s", region)
 	}
@@ -1142,3 +1199,8 @@ func getMgsEndpoint(context context.T, region string) (string, error) {
 	endpointBuilder.WriteString(hostName)
 	return endpointBuilder.String(), nil
 }
+
+// getNonRetryableDataChannelErrors returns list of non retryable errors for data channel retry strategy
+func getNonRetryableDataChannelErrors() []string {
+	return []string{mgsConfig.SessionAlreadyTerminatedError}
+}
