@@ -19,10 +19,15 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
+	"io"
 	"os"
+	"os/exec"
 	"os/signal"
 	"path/filepath"
+	"regexp"
 	"runtime/debug"
+	"strconv"
+	"strings"
 	"syscall"
 	"time"
 	"unicode/utf8"
@@ -39,6 +44,7 @@ import (
 	mgsConfig "github.com/aws/amazon-ssm-agent/agent/session/config"
 	mgsContracts "github.com/aws/amazon-ssm-agent/agent/session/contracts"
 	"github.com/aws/amazon-ssm-agent/agent/session/datachannel"
+	"github.com/aws/amazon-ssm-agent/agent/session/shell/constants"
 	"github.com/aws/amazon-ssm-agent/agent/session/shell/execcmd"
 	"github.com/aws/amazon-ssm-agent/agent/task"
 	"github.com/aws/aws-sdk-go/service/cloudwatchlogs"
@@ -46,14 +52,19 @@ import (
 
 // Plugin is the type for the plugin.
 type ShellPlugin struct {
-	context     context.T
-	name        string
-	stdin       *os.File
-	stdout      *os.File
-	execCmd     execcmd.IExecCmd
-	runAsUser   string
-	dataChannel datachannel.IDataChannel
-	logger      logger
+	context        context.T
+	name           string
+	stdin          *os.File
+	stdout         *os.File
+	stdoutPipe     io.Reader
+	stderrPipe     io.Reader
+	execCmd        execcmd.IExecCmd
+	runAsUser      string
+	dataChannel    datachannel.IDataChannel
+	logger         logger
+	separateOutput bool
+	stdoutPrefix   string
+	stderrPrefix   string
 }
 
 // logger is used for storing the information related to logging of session data to S3/CW
@@ -65,6 +76,7 @@ type logger struct {
 	ptyTerminated               chan bool
 	cloudWatchStreamingFinished chan bool
 	streamLogsToCloudWatch      bool
+	writeToIpcFile              bool
 	s3Util                      s3util.IAmazonS3Util
 	cwl                         cloudwatchlogsinterface.ICloudWatchLogsService
 }
@@ -74,6 +86,8 @@ type IShellPlugin interface {
 	InputStreamMessageHandler(log log.T, streamDataMessage mgsContracts.AgentMessage) error
 }
 
+const separateOutputStreamPrefixRegex = "^[0-9a-zA-Z\r\n_:-]{0,30}$"
+
 // NewPlugin returns a new instance of the Shell Plugin
 func NewPlugin(context context.T, name string) (*ShellPlugin, error) {
 	var plugin = ShellPlugin{
@@ -126,6 +140,37 @@ func (p *ShellPlugin) validate(config agentContracts.Configuration) error {
 	return nil
 }
 
+// validPrefix checks whether the given prefix string is valid.
+func (p *ShellPlugin) validPrefix(prefix string) bool {
+	prefixRegex := regexp.MustCompile(separateOutputStreamPrefixRegex)
+	return prefixRegex.MatchString(prefix)
+}
+
+// setSeparateOutputStreamProperties validates separateOutputStream properties and set them to shell plugin context.
+func (p *ShellPlugin) setSeparateOutputStreamProperties(shellProps mgsContracts.ShellProperties) error {
+	separateOutput, err := constants.GetSeparateOutputStream(shellProps)
+	if err != nil {
+		return fmt.Errorf("fail to get separateOutPutStream property: %v", err)
+	}
+	p.separateOutput = separateOutput
+
+	if p.separateOutput {
+		stdoutPrefix := constants.GetStdOutSeparatorPrefix(shellProps)
+		if !p.validPrefix(stdoutPrefix) {
+			return fmt.Errorf("invalid stdoutSeparatorPrefix %v", stdoutPrefix)
+		}
+		p.stdoutPrefix = stdoutPrefix
+
+		stderrPrefix := constants.GetStdErrSeparatorPrefix(shellProps)
+		if !p.validPrefix(stderrPrefix) {
+			return fmt.Errorf("invalid stderrSeparatorPrefix %v", stderrPrefix)
+		}
+		p.stderrPrefix = stderrPrefix
+	}
+
+	return nil
+}
+
 // Execute starts pseudo terminal.
 // It reads incoming message from data channel and writes to pty.stdin.
 // It reads message from pty.stdout and writes to data channel
@@ -193,18 +238,26 @@ func (p *ShellPlugin) execute(config agentContracts.Configuration,
 		return
 	}
 
+	if appconfig.PluginNameNonInteractiveCommands == p.name {
+		if err := p.setSeparateOutputStreamProperties(shellProps); err != nil {
+			output.SetExitCode(appconfig.ErrorExitCode)
+			output.SetStatus(agentContracts.ResultStatusFailed)
+			sessionPluginResultOutput.Output = err.Error()
+			output.SetOutput(sessionPluginResultOutput)
+			log.Errorf("SeparateOutputStream properties validation failed, err: %s", err)
+			return
+		}
+	}
 	// Catch signals and send a signal to the "sigs" chan if it triggers
 	sigs := make(chan os.Signal, 1)
 	signal.Notify(sigs, syscall.SIGINT, syscall.SIGHUP, syscall.SIGTERM, syscall.SIGQUIT)
 
-	var isSessionCancelled = false
 	// Setup cancellation flag for accepting TerminateSession requests and idle session timeout scenarios
 	cancelled := make(chan bool, 1)
 	go func() {
 		sig := <-sigs
 		log.Infof("caught signal to terminate: %v", sig)
 		cancelled <- true
-		isSessionCancelled = true
 	}()
 
 	// Get the command executor, which is either pseudo terminal or exec.Cmd depending on the plugin type
@@ -212,34 +265,41 @@ func (p *ShellPlugin) execute(config agentContracts.Configuration,
 		errorString := fmt.Errorf("Unable to start command: %s\n", err)
 		log.Error(errorString)
 		time.Sleep(2 * time.Second)
+		if appconfig.PluginNameNonInteractiveCommands == p.name {
+			// Error started before exec.cmd starts needs to be explicitly propagated to data channel.
+			p.sendErrorToDataChannel(log, errorString.Error())
+		}
 		output.MarkAsFailed(errorString)
 		return
 	}
 
 	// Create ipcFile used for logging session data temporarily on disk
-	ipcFile, err := os.Create(p.logger.ipcFilePath)
+	ipcFile, err := p.createIpcFile()
 	if err != nil {
 		errorString := fmt.Errorf("encountered an error while creating file %s: %s", p.logger.ipcFilePath, err)
 		log.Error(errorString)
 		output.MarkAsFailed(errorString)
 		return
 	}
-	defer ipcFile.Close()
+	defer func() {
+		if closeErr := ipcFile.Close(); closeErr != nil {
+			log.Warnf("error occurred while closing ipcFile, %v", closeErr)
+		}
+	}()
 
 	go func() {
 		cancelState := cancelFlag.Wait()
 		if cancelFlag.Canceled() {
 			cancelled <- true
-			isSessionCancelled = true
 			log.Debug("Cancel flag set to cancelled in session")
 		}
 		log.Debugf("Cancel flag set to %v in session", cancelState)
 	}()
 
 	if appconfig.PluginNameNonInteractiveCommands == p.name {
-		p.executeCommandsWithExec(config, cancelled, isSessionCancelled, output, ipcFile)
+		p.executeCommandsWithExec(config, cancelled, cancelFlag, output, ipcFile)
 	} else {
-		p.executeCommandsWithPty(config, cancelled, isSessionCancelled, output, ipcFile)
+		p.executeCommandsWithPty(config, cancelled, cancelFlag, output, ipcFile)
 	}
 
 	// Finish logger activity like uploading logs to S3/CW
@@ -248,16 +308,25 @@ func (p *ShellPlugin) execute(config agentContracts.Configuration,
 	log.Debug("Shell session execution complete")
 }
 
+// Creates ipc temp file
+func (p *ShellPlugin) createIpcFile() (*os.File, error) {
+	if !p.logger.writeToIpcFile {
+		return nil, nil
+	}
+	ipcFile, err := os.Create(p.logger.ipcFilePath)
+	return ipcFile, err
+}
+
 // Executes command in pseudo terminal with pty
 func (p *ShellPlugin) executeCommandsWithPty(config agentContracts.Configuration,
 	cancelled chan bool,
-	isSessionCancelled bool,
+	cancelFlag task.CancelFlag,
 	output iohandler.IOHandler,
 	ipcFile *os.File) {
 
 	log := p.context.Log()
 
-	writePumpDone := p.setupRoutineToWriteCommandOutput(log, ipcFile)
+	writePumpDone := p.setupRoutineToWriteCommandOutput(log, ipcFile, 1)
 
 	log.Infof("Plugin %s started", p.name)
 
@@ -324,7 +393,7 @@ func (p *ShellPlugin) executeCommandsWithPty(config agentContracts.Configuration
 			output.SetExitCode(appconfig.SuccessExitCode)
 			output.SetStatus(agentContracts.ResultStatusSuccess)
 		}
-		if isSessionCancelled {
+		if cancelFlag.Canceled() {
 			log.Errorf("The cancellation failed to stop the session.")
 		}
 	}
@@ -333,7 +402,7 @@ func (p *ShellPlugin) executeCommandsWithPty(config agentContracts.Configuration
 // Execute single command in non-interactive mode with exec.Cmd
 func (p *ShellPlugin) executeCommandsWithExec(config agentContracts.Configuration,
 	cancelled chan bool,
-	isSessionCancelled bool,
+	cancelFlag task.CancelFlag,
 	output iohandler.IOHandler,
 	ipcFile *os.File) {
 
@@ -341,25 +410,160 @@ func (p *ShellPlugin) executeCommandsWithExec(config agentContracts.Configuratio
 
 	log.Infof("Plugin %s started", p.name)
 
-	if err := p.execCmd.Start(); err != nil {
-		errorString := fmt.Errorf("Error occurred starting the command: %s\n", err)
-		log.Error(errorString)
-		output.MarkAsFailed(errorString)
-		return
-	}
-
 	// CW streaming logs is disabled for NonInteractiveCommands plugin, which is by far the only session plugin that uses exec.Cmd.
 	// However, leaving the startStreamingLogs call path here in case future session plugins use exec.Cmd differently and need streaming logs.
 	p.startStreamingLogs(ipcFile, config)
 
+	if p.separateOutput {
+		if err := p.processCommandsWithOutputStreamSeparate(cancelled, cancelFlag, output, ipcFile); err != nil {
+			p.sendErrorToDataChannel(log, err.Error())
+			// Call datachannel PrepareToCloseChannel so all messages in the buffer are sent
+			p.dataChannel.PrepareToCloseChannel(log)
+			// Send session status as Terminating to service on completing command execution
+			if err := p.dataChannel.SendAgentSessionStateMessage(log, mgsContracts.Terminating); err != nil {
+				log.Errorf("Unable to send AgentSessionState message with session status %s. %v", mgsContracts.Terminating, err)
+			}
+		}
+	} else {
+		if err := p.processCommandsWithExec(cancelled, cancelFlag, output, ipcFile); err != nil {
+			p.sendErrorToDataChannel(log, err.Error())
+			// Call datachannel PrepareToCloseChannel so all messages in the buffer are sent
+			p.dataChannel.PrepareToCloseChannel(log)
+			// Send session status as Terminating to service on completing command execution
+			if err := p.dataChannel.SendAgentSessionStateMessage(log, mgsContracts.Terminating); err != nil {
+				log.Errorf("Unable to send AgentSessionState message with session status %s. %v", mgsContracts.Terminating, err)
+			}
+		}
+		p.cleanupOutputFile(log, config)
+	}
+}
+
+// Handle go routines between session termination and command execution with exec.Cmd
+func (p *ShellPlugin) processCommandsWithOutputStreamSeparate(cancelled chan bool,
+	cancelFlag task.CancelFlag,
+	output iohandler.IOHandler,
+	ipcFile *os.File) (err error) {
+
+	log := p.context.Log()
+
+	writeStdOutDone := p.setupRoutineToWriteCmdPipelineOutput(log, ipcFile, false)
+	writeStdErrDone := p.setupRoutineToWriteCmdPipelineOutput(log, ipcFile, true)
+
+	if err := p.execCmd.Start(); err != nil {
+		errorString := fmt.Errorf("Error occurred starting the command: %s\n", err)
+		log.Error(errorString)
+		commandExitCode := appconfig.ErrorExitCode
+		p.sendExitCode(log, ipcFile, commandExitCode)
+		output.MarkAsFailed(errorString)
+		return err
+	}
+
+	// Wait for session to be completed/cancelled/interrupted
+	cmdWaitDone := make(chan error, 1)
+	cmdExitCode := make(chan int, 1)
+	writeStdOutResult, writeStdErrResult := appconfig.ErrorExitCode, appconfig.ErrorExitCode
+
+	go func() {
+		defer func() {
+			if err := recover(); err != nil {
+				log.Errorf("Write Stdout thread crashed with message: %v\n", err)
+				log.Errorf("Stacktrace:\n%s", debug.Stack())
+			}
+		}()
+		log.Debugf("Start separate go routine to wait for command to complete. Pid: %v", p.execCmd.Pid())
+		writeStdOutResult, writeStdErrResult = <-writeStdOutDone, <-writeStdErrDone
+		log.Debugf("writeStdOutResult: %v, writeStdErrResult: %v", writeStdOutResult, writeStdErrResult)
+		close(writeStdOutDone)
+		close(writeStdErrDone)
+
+		err := p.execCmd.Wait()
+		if err != nil {
+			if exiterr, ok := err.(*exec.ExitError); ok {
+				log.Infof("Command Exit Status: %d", exiterr.ExitCode())
+				cmdExitCode <- exiterr.ExitCode()
+
+			} else {
+				log.Errorf("Failed to get exit code, set it to %v", appconfig.ErrorExitCode)
+				cmdExitCode <- appconfig.ErrorExitCode
+			}
+		} else {
+			log.Infof("Command success with exit status 0")
+			cmdExitCode <- appconfig.SuccessExitCode
+		}
+		cmdWaitDone <- err
+	}()
+
+	select {
+	case <-cancelled:
+		log.Debug("Session cancelled. Attempting to stop the command execution.")
+		if err := p.execCmd.Kill(); err != nil {
+			log.Errorf("unable to terminate command execution process %s: %v", p.execCmd.Pid(), err)
+		}
+		output.SetExitCode(appconfig.SuccessExitCode)
+		output.SetStatus(agentContracts.ResultStatusSuccess)
+		log.Info("The session was cancelled")
+
+	case cmdWaitErr := <-cmdWaitDone:
+		if cmdWaitErr != nil {
+			log.Errorf("received error when waiting for command to complete: %v", cmdWaitErr)
+		}
+		if cancelFlag.Canceled() {
+			log.Errorf("the cancellation failed to stop the session.")
+		}
+
+		if writeStdOutResult == appconfig.SuccessExitCode && writeStdErrResult == appconfig.SuccessExitCode {
+			log.Debugf("Writing session plugin output is done. Exit code: 0.")
+			output.SetExitCode(appconfig.SuccessExitCode)
+			output.SetStatus(agentContracts.ResultStatusSuccess)
+		} else {
+			log.Debugf("Writing session plugin output is done. Exit code: 1.")
+			output.SetExitCode(appconfig.ErrorExitCode)
+			output.SetStatus(agentContracts.ResultStatusFailed)
+		}
+		commandExitCode := <-cmdExitCode
+		close(cmdExitCode)
+		log.Infof("The session commandExitCode %d", commandExitCode)
+		p.sendExitCode(log, ipcFile, commandExitCode)
+	}
+
+	// Call datachannel PrepareToCloseChannel so all messages in the buffer are sent
+	p.dataChannel.PrepareToCloseChannel(log)
+
+	// Send session status as Terminating to service on completing command execution
+	if err := p.dataChannel.SendAgentSessionStateMessage(log, mgsContracts.Terminating); err != nil {
+		log.Errorf("Unable to send AgentSessionState message with session status %s. %v", mgsContracts.Terminating, err)
+	}
+
+	return nil
+}
+
+// Handle go routines between session termination and command execution with exec.Cmd
+func (p *ShellPlugin) processCommandsWithExec(cancelled chan bool,
+	cancelFlag task.CancelFlag,
+	output iohandler.IOHandler,
+	ipcFile *os.File) (err error) {
+
+	log := p.context.Log()
+
+	if err := p.execCmd.Start(); err != nil {
+		errorString := fmt.Errorf("Error occurred starting the command: %s\n", err)
+		log.Error(errorString)
+		output.MarkAsFailed(errorString)
+		return err
+	}
+
 	// Wait for session to be completed/cancelled/interrupted
-	var isCmdWaitSuccess = false
 	cmdWaitDone := make(chan error, 1)
 	go func() {
+		defer func() {
+			if r := recover(); r != nil {
+				log.Errorf("Process commands with exec panic: %v", r)
+				log.Errorf("Stacktrace:\n%s", debug.Stack())
+			}
+		}()
 		log.Debugf("Start separate go routine to wait for command to complete. Pid: %v", p.execCmd.Pid())
 		err := p.execCmd.Wait()
 		cmdWaitDone <- err
-		isCmdWaitSuccess = err == nil
 	}()
 
 	select {
@@ -374,53 +578,56 @@ func (p *ShellPlugin) executeCommandsWithExec(config agentContracts.Configuratio
 		log.Info("The session was cancelled")
 
 	case cmdWaitErr := <-cmdWaitDone:
-		if !isCmdWaitSuccess {
+		if cmdWaitErr != nil {
 			log.Errorf("received error when waiting for command to complete: %v", cmdWaitErr)
 		}
-		if isSessionCancelled {
+		if cancelFlag.Canceled() {
 			log.Errorf("the cancellation failed to stop the session.")
 		}
 	}
 
-	// If session is not cancelled, start go routine to read output from disk and write to data channel
-	if !isSessionCancelled {
-		writePumpDone := p.setupRoutineToWriteCommandOutput(log, ipcFile)
+	writePumpDone := p.setupRoutineToWriteCommandOutput(log, ipcFile, 0)
 
-		select {
-		case <-cancelled:
-			log.Debug("Session cancelled. Attempting to stop the command execution.")
-			if err := p.execCmd.Kill(); err != nil {
-				log.Errorf("unable to terminate command execution process %s: %v", p.execCmd.Pid(), err)
-			}
-			errorCode := 0
-			output.SetExitCode(errorCode)
+	select {
+	case <-cancelled:
+		log.Debug("Session cancelled. Attempting to stop the command execution.")
+		if err := p.execCmd.Kill(); err != nil {
+			log.Errorf("unable to terminate command execution process %s: %v", p.execCmd.Pid(), err)
+		}
+		errorCode := 0
+		output.SetExitCode(errorCode)
+		output.SetStatus(agentContracts.ResultStatusSuccess)
+		log.Info("The session was cancelled")
+
+	case exitCode := <-writePumpDone:
+		log.Debugf("Writing command output is done. Exit code: %v.", exitCode)
+		if exitCode == 1 {
+			output.SetExitCode(appconfig.ErrorExitCode)
+			output.SetStatus(agentContracts.ResultStatusFailed)
+		} else {
+			output.SetExitCode(appconfig.SuccessExitCode)
 			output.SetStatus(agentContracts.ResultStatusSuccess)
-			log.Info("The session was cancelled")
+		}
 
-		case exitCode := <-writePumpDone:
-			log.Debugf("Writing command output is done. Exit code: %v.", exitCode)
-			if isSessionCancelled {
-				log.Errorf("The cancellation failed to stop the session.")
-			}
-			if exitCode == 1 {
-				output.SetExitCode(appconfig.ErrorExitCode)
-				output.SetStatus(agentContracts.ResultStatusFailed)
-				return
-			}
-			if isCmdWaitSuccess {
-				output.SetExitCode(appconfig.SuccessExitCode)
-				output.SetStatus(agentContracts.ResultStatusSuccess)
-			} else {
-				output.SetExitCode(appconfig.ErrorExitCode)
-				output.SetStatus(agentContracts.ResultStatusFailed)
-			}
+		// Call datachannel PrepareToCloseChannel so all messages in the buffer are sent
+		p.dataChannel.PrepareToCloseChannel(log)
+
+		// Send session status as Terminating to service on completing command execution
+		if err := p.dataChannel.SendAgentSessionStateMessage(log, mgsContracts.Terminating); err != nil {
+			log.Errorf("Unable to send AgentSessionState message with session status %s. %v", mgsContracts.Terminating, err)
 		}
 	}
-	p.cleanupOutputFile(log, config)
+
+	return nil
 }
 
 // initializeLogger initializes plugin logger to be used for s3/cw logging
 func (p *ShellPlugin) initializeLogger(log log.T, config agentContracts.Configuration) {
+	p.logger.writeToIpcFile = !(config.OutputS3BucketName == "" && config.CloudWatchLogGroup == "" && p.context.AppConfig().Ssm.SessionLogsDestination == appconfig.SessionLogsDestinationNone)
+	if p.logger.writeToIpcFile && p.context.AppConfig().Ssm.SessionLogsDestination == appconfig.SessionLogsDestinationNone {
+		log.Warn("Overriding SessionLogsDestination: \"none\" since logging session data to CloudWatch or S3 is enabled.")
+	}
+
 	if config.OutputS3BucketName != "" {
 		var err error
 		p.logger.s3Util, err = s3util.NewAmazonS3Util(p.context, config.OutputS3BucketName)
@@ -460,22 +667,125 @@ func (p *ShellPlugin) uploadShellSessionLogsToS3(log log.T, s3UploaderUtil s3uti
 }
 
 // Set up go routine to write command output to data channel
-func (p *ShellPlugin) setupRoutineToWriteCommandOutput(log log.T, ipcFile *os.File) chan int {
+func (p *ShellPlugin) setupRoutineToWriteCommandOutput(log log.T, ipcFile *os.File, initialWaitSecond int) chan int {
 	log.Debugf("Start separate go routine to read from command output and write to data channel")
 
 	done := make(chan int, 1)
 	go func() {
-		done <- p.writePump(log, ipcFile)
+		done <- p.writePump(log, ipcFile, initialWaitSecond)
 	}()
 
 	return done
 }
 
+// Set up go routine to write command output to data channel
+func (p *ShellPlugin) setupRoutineToWriteCmdPipelineOutput(log log.T, ipcFile *os.File, isStderr bool) chan int {
+	done := make(chan int, 1)
+
+	go func() {
+		defer func() {
+			if err := recover(); err != nil {
+				log.Errorf("Write pipeline output crashed with message: %v\n", err)
+				log.Errorf("Stacktrace:\n%s", debug.Stack())
+			}
+		}()
+
+		var pipe io.Reader
+		var unprocessedBuf bytes.Buffer
+		var outputPrefix string
+		prefix := make([]byte, 128)
+		payloadType := mgsContracts.Output
+
+		if isStderr {
+			pipe = p.stderrPipe
+			outputPrefix = p.stderrPrefix
+			payloadType = mgsContracts.StdErr
+		} else {
+			pipe = p.stdoutPipe
+			outputPrefix = p.stdoutPrefix
+		}
+
+		prefixLen := len(outputPrefix)
+		if prefixLen > 0 {
+			r := strings.NewReader(outputPrefix)
+			len, err := r.Read(prefix)
+			if err != nil {
+				log.Debugf("Failed to read prefix: %s", err)
+			}
+			prefixLen = len
+		}
+
+		prefix = prefix[:prefixLen]
+		outputBytes := make([]byte, mgsConfig.StreamDataPayloadSize-prefixLen)
+		needPrefix := true
+		for {
+			if p.dataChannel.IsActive() {
+				outputBytesLen, err := pipe.Read(outputBytes)
+				if err == io.EOF {
+					log.Debugf("Pipeline closed, finish pipeline reading. Is StdErr pipe: %t", isStderr)
+					done <- appconfig.SuccessExitCode
+					break
+				} else if err != nil {
+					log.Errorf("Failed to read from command output pipeline: %s", err)
+					done <- appconfig.ErrorExitCode
+					break
+				}
+				// Add prefix for first none empty frame, later decide it based on content of last character
+				if needPrefix && outputBytesLen > 0 {
+					unprocessedBuf.Write(prefix)
+				}
+				if outputBytesLen > 0 && outputBytes[outputBytesLen-1] != '\n' {
+					needPrefix = false
+				} else {
+					needPrefix = true
+				}
+				// unprocessedBuf contains incomplete utf8 encoded unicode bytes returned after processing of stdoutBytes
+				if unprocessedBuf, err = p.processStdoutData(log, outputBytes, outputBytesLen, unprocessedBuf, ipcFile, payloadType); err != nil {
+					log.Errorf("Error processing command pipeline output data, %v", err)
+					done <- appconfig.ErrorExitCode
+					break
+				}
+			} else {
+				log.Errorf("Data Channel not in active status")
+				done <- appconfig.ErrorExitCode
+				break
+			}
+		}
+	}()
+
+	return done
+}
+
+// Write command exit code to data channel
+func (p *ShellPlugin) sendExitCode(log log.T, ipcFile *os.File, exitCode int) error {
+	var unprocessedBuf bytes.Buffer
+	outputBytes := make([]byte, 128)
+
+	output := "EXIT_CODE: " + strconv.Itoa(exitCode)
+	r := strings.NewReader(output)
+	outputBytesLen, err := r.Read(outputBytes)
+	if err != nil {
+		log.Debugf("Failed to read prefix for exit code: %s", err)
+		return err
+	}
+	log.Infof("Sending exit code: %d", exitCode)
+
+	if p.dataChannel.IsActive() {
+		if unprocessedBuf, err = p.processStdoutData(log, outputBytes, outputBytesLen, unprocessedBuf, ipcFile, mgsContracts.ExitCode); err != nil {
+			log.Errorf("Error processing command pipeline output data, %v", err)
+		}
+	} else {
+		return fmt.Errorf("failed to send exit code as data channel closed")
+	}
+
+	return nil
+}
+
 // writePump reads from pty stdout and writes to data channel.
-func (p *ShellPlugin) writePump(log log.T, ipcFile *os.File) (errorCode int) {
+func (p *ShellPlugin) writePump(log log.T, ipcFile *os.File, initialWaitSecond int) (errorCode int) {
 	defer func() {
 		if err := recover(); err != nil {
-			fmt.Println("WritePump thread crashed with message: \n", err)
+			log.Errorf("WritePump thread crashed with message: %v\n", err)
 			log.Errorf("Stacktrace:\n%s", debug.Stack())
 		}
 	}()
@@ -484,19 +794,19 @@ func (p *ShellPlugin) writePump(log log.T, ipcFile *os.File) (errorCode int) {
 	reader := bufio.NewReader(p.stdout)
 
 	// Wait for all input commands to run.
-	time.Sleep(time.Second)
+	time.Sleep(time.Duration(initialWaitSecond) * time.Second)
 
 	var unprocessedBuf bytes.Buffer
 	for {
 		if p.dataChannel.IsActive() {
 			stdoutBytesLen, err := reader.Read(stdoutBytes)
 			if err != nil {
-				log.Debugf("Failed to read from pty master: %s", err)
+				log.Debugf("Failed to read from command output: %s", err)
 				return appconfig.SuccessExitCode
 			}
 
 			// unprocessedBuf contains incomplete utf8 encoded unicode bytes returned after processing of stdoutBytes
-			if unprocessedBuf, err = p.processStdoutData(log, stdoutBytes, stdoutBytesLen, unprocessedBuf, ipcFile); err != nil {
+			if unprocessedBuf, err = p.processStdoutData(log, stdoutBytes, stdoutBytesLen, unprocessedBuf, ipcFile, mgsContracts.Output); err != nil {
 				log.Errorf("Error processing stdout data, %v", err)
 				return appconfig.ErrorExitCode
 			}
@@ -513,7 +823,8 @@ func (p *ShellPlugin) processStdoutData(
 	stdoutBytes []byte,
 	stdoutBytesLen int,
 	unprocessedBuf bytes.Buffer,
-	file *os.File) (bytes.Buffer, error) {
+	file *os.File,
+	payloadType mgsContracts.PayloadType) (bytes.Buffer, error) {
 
 	// append stdoutBytes to unprocessedBytes and then read rune from appended bytes to send it over websocket channel
 	unprocessedBytes := unprocessedBuf.Bytes()
@@ -549,12 +860,14 @@ func (p *ShellPlugin) processStdoutData(
 		i += stdoutRuneLen
 	}
 
-	if err := p.dataChannel.SendStreamDataMessage(log, mgsContracts.Output, processedBuf.Bytes()); err != nil {
+	if err := p.dataChannel.SendStreamDataMessage(log, payloadType, processedBuf.Bytes()); err != nil {
 		return processedBuf, fmt.Errorf("unable to send stream data message: %s", err)
 	}
 
-	if _, err := file.Write(processedBuf.Bytes()); err != nil {
-		return processedBuf, fmt.Errorf("encountered an error while writing to file: %s", err)
+	if p.logger.writeToIpcFile {
+		if _, err := file.Write(processedBuf.Bytes()); err != nil {
+			return processedBuf, fmt.Errorf("encountered an error while writing to file: %s", err)
+		}
 	}
 
 	// return incomplete utf8 encoded unicode bytes to be processed with next batch of stdoutBytes
@@ -681,3 +994,10 @@ func (p *ShellPlugin) cleanupOutputFile(log log.T, config agentContracts.Configu
 		log.Debugf("Unable to clean up output file, %v", err)
 	}
 }
+
+func (p *ShellPlugin) sendErrorToDataChannel(log log.T, errorString string) {
+	time.Sleep(1 * time.Second)
+	if dataChannelError := p.dataChannel.SendStreamDataMessage(log, mgsContracts.StdErr, []byte(errorString)); dataChannelError != nil {
+		log.Errorf("Unable to send error message to data channel: %v", dataChannelError)
+	}
+}
