@@ -15,7 +15,6 @@ package ssm
 
 import (
 	"fmt"
-	"net/http"
 	"runtime"
 	"time"
 
@@ -67,7 +66,7 @@ type Service interface {
 	GetDocument(log log.T, docName string, docVersion string) (response *ssm.GetDocumentOutput, err error)
 	DeleteDocument(log log.T, instanceID string) (response *ssm.DeleteDocumentOutput, err error)
 	DescribeAssociation(log log.T, instanceID string, docName string) (response *ssm.DescribeAssociationOutput, err error)
-	UpdateInstanceInformation(log log.T, agentVersion, agentStatus, agentName string) (response *ssm.UpdateInstanceInformationOutput, err error)
+	UpdateInstanceInformation(log log.T, agentVersion, agentStatus, agentName string, availabilityZone string, availabilityZoneId string, ssmConnectionChannel string) (response *ssm.UpdateInstanceInformationOutput, err error)
 	UpdateEmptyInstanceInformation(log log.T, agentVersion, agentName string) (response *ssm.UpdateInstanceInformationOutput, err error)
 	GetParameters(log log.T, paramNames []string) (response *ssm.GetParametersOutput, err error)
 	GetDecryptedParameters(log log.T, paramNames []string) (response *ssm.GetParametersOutput, err error)
@@ -88,25 +87,15 @@ func NewService(context context.T) Service {
 		ssmStopPolicy = sdkutil.NewStopPolicy("ssmService", 10)
 	}
 
-	awsConfig := sdkutil.AwsConfig(context)
+	awsConfig := sdkutil.AwsConfig(context, "ssm")
 	// parse appConfig overrides
 	appConfig := context.AppConfig()
 	if appConfig.Ssm.Endpoint != "" {
 		awsConfig.Endpoint = &appConfig.Ssm.Endpoint
-	} else {
-		if defaultEndpoint := context.Identity().GetDefaultEndpoint("ssm"); defaultEndpoint != "" {
-			awsConfig.Endpoint = &defaultEndpoint
-		}
-	}
-	if appConfig.Agent.Region != "" {
-		awsConfig.Region = &appConfig.Agent.Region
 	}
 
-	// TODO: test hook, can be removed before release
-	// this is to skip ssl verification for the beta self signed certs
-	if appConfig.Ssm.InsecureSkipVerify {
-		tlsConfig := awsConfig.HTTPClient.Transport.(*http.Transport).TLSClientConfig
-		tlsConfig.InsecureSkipVerify = true
+	if appConfig.Agent.Region != "" {
+		awsConfig.Region = &appConfig.Agent.Region
 	}
 
 	sess := session.New(awsConfig)
@@ -128,7 +117,7 @@ func makeAwsStrings(strings []string) []*string {
 	return out
 }
 
-//ListAssociations calls the ListAssociations SSM API.
+// ListAssociations calls the ListAssociations SSM API.
 func (svc *sdkService) ListAssociations(log log.T, instanceID string) (response *ssm.ListAssociationsOutput, err error) {
 	params := ssm.ListAssociationsInput{
 		AssociationFilterList: []*ssm.AssociationFilter{
@@ -148,7 +137,7 @@ func (svc *sdkService) ListAssociations(log log.T, instanceID string) (response
 	return
 }
 
-//ListInstanceAssociations calls the ListAssociations SSM API.
+// ListInstanceAssociations calls the ListAssociations SSM API.
 func (svc *sdkService) ListInstanceAssociations(log log.T, instanceID string, nextToken *string) (response *ssm.ListInstanceAssociationsOutput, err error) {
 	params := ssm.ListInstanceAssociationsInput{
 		InstanceId: &instanceID,
@@ -204,7 +193,7 @@ func (svc *sdkService) PutComplianceItems(
 	return
 }
 
-//UpdateInstanceAssociationStatus calls the ListAssociations SSM API.
+// UpdateInstanceAssociationStatus calls the ListAssociations SSM API.
 func (svc *sdkService) UpdateInstanceAssociationStatus(log log.T, associationID string, instanceID string, executionResult *ssm.InstanceAssociationExecutionResult) (response *ssm.UpdateInstanceAssociationStatusOutput, err error) {
 	params := ssm.UpdateInstanceAssociationStatusInput{
 		InstanceId:      &instanceID,
@@ -221,7 +210,7 @@ func (svc *sdkService) UpdateInstanceAssociationStatus(log log.T, associationID
 	return
 }
 
-//UpdateAssociationStatus calls the UpdateAssociationStatus SSM API.
+// UpdateAssociationStatus calls the UpdateAssociationStatus SSM API.
 func (svc *sdkService) UpdateAssociationStatus(
 	log log.T,
 	instanceID string,
@@ -242,18 +231,24 @@ func (svc *sdkService) UpdateAssociationStatus(
 	return
 }
 
-//UpdateInstanceInformation calls the UpdateInstanceInformation SSM API.
+// UpdateInstanceInformation calls the UpdateInstanceInformation SSM API.
 func (svc *sdkService) UpdateInstanceInformation(
 	log log.T,
 	agentVersion,
 	agentStatus,
 	agentName string,
+	availabilityZone string,
+	availabilityZoneId string,
+	ssmConnectionChannel string,
 ) (response *ssm.UpdateInstanceInformationOutput, err error) {
 
 	params := ssm.UpdateInstanceInformationInput{
-		AgentName:    aws.String(agentName),
-		AgentStatus:  aws.String(agentStatus),
-		AgentVersion: aws.String(agentVersion),
+		AgentName:            aws.String(agentName),
+		AgentStatus:          aws.String(agentStatus),
+		AgentVersion:         aws.String(agentVersion),
+		AvailabilityZone:     aws.String(availabilityZone),
+		AvailabilityZoneId:   aws.String(availabilityZoneId),
+		SSMConnectionChannel: aws.String(ssmConnectionChannel),
 	}
 
 	goOS := runtime.GOOS
@@ -307,7 +302,7 @@ func (svc *sdkService) UpdateInstanceInformation(
 	return
 }
 
-//UpdateEmptyInstanceInformation calls the UpdateInstanceInformation SSM API with an empty ping.
+// UpdateEmptyInstanceInformation calls the UpdateInstanceInformation SSM API with an empty ping.
 func (svc *sdkService) UpdateEmptyInstanceInformation(
 	log log.T,
 	agentVersion,
@@ -358,7 +353,7 @@ func (svc *sdkService) CreateDocument(log log.T, docName string, docContent stri
 	return
 }
 
-//GetDocument calls the GetDocument SSM API to retrieve document with given document name
+// GetDocument calls the GetDocument SSM API to retrieve document with given document name
 func (svc *sdkService) GetDocument(log log.T, docName string, docVersion string) (response *ssm.GetDocumentOutput, err error) {
 	params := ssm.GetDocumentInput{
 		Name: aws.String(docName),
@@ -377,7 +372,7 @@ func (svc *sdkService) GetDocument(log log.T, docName string, docVersion string)
 	return
 }
 
-//DescribeAssociation calls the DescribeAssociation SSM API to retrieve parameters information
+// DescribeAssociation calls the DescribeAssociation SSM API to retrieve parameters information
 func (svc *sdkService) DescribeAssociation(log log.T, instanceID string, docName string) (response *ssm.DescribeAssociationOutput, err error) {
 	params := ssm.DescribeAssociationInput{
 		InstanceId: aws.String(instanceID),
