@@ -11,6 +11,7 @@
 // either express or implied. See the License for the specific language governing
 // permissions and limitations under the License.
 //
+//go:build windows
 // +build windows
 
 // Package startup implements startup plugin processor
@@ -26,6 +27,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/aws/amazon-ssm-agent/agent/appconfig"
 	"github.com/aws/amazon-ssm-agent/agent/log"
 	"github.com/aws/amazon-ssm-agent/agent/platform"
 	"github.com/aws/amazon-ssm-agent/agent/startup/model"
@@ -60,6 +62,10 @@ const (
 	PvName            = "Name"
 	PvVersionProperty = "Version"
 
+	// NitroEnclavesEntity Properties
+	NitroEnclavesName            = "Name"
+	NitroEnclavesVersionProperty = "Version"
+
 	// PnpEntity Properties
 	deviceIDProperty = "DeviceID"
 	serviceProperty  = "Service"
@@ -91,6 +97,9 @@ const (
 	// PS command to get AWS PV package entry from registry HKLM:\SOFTWARE\Amazon\PVDriver
 	getPvPackageVersionCmd = "Get-ItemProperty -Path 'HKLM:\\SOFTWARE\\Amazon\\PVDriver'"
 
+	// PS command to get AWS Nitro Enclaves package entry from registry HKLM:\SOFTWARE\Amazon\AwsNitroEnclaves
+	getNitroEnclavesPackageVersionCmd = "Get-ItemProperty -Path 'HKLM:\\SOFTWARE\\Amazon\\AwsNitroEnclaves'"
+
 	// PS command to get AWS PV Storage Host Adapter entry shown in Device Manager
 	getPvDriverPnpEntityCmd = "Get-CimInstance Win32_PnPEntity | Where-Object { $_.Service -eq 'xenvbd' }"
 
@@ -254,6 +263,12 @@ func (p *Processor) ExecuteTasks() (err error) {
 		sp.WritePort(fmt.Sprintf("Driver: AWS PV Driver Package v%v", pvPackageInfo.Version))
 	}
 
+	nitroEnclavesPackageInfo, NitroEnclavesError := getAWSNitroEnclavesPackageInfo(log)
+	// write AWS Nitro Enclaves Package version to serial port if exists
+	if NitroEnclavesError == nil {
+		sp.WritePort(fmt.Sprintf("Driver: AWS Nitro Enclaves Package v%v", nitroEnclavesPackageInfo.Version))
+	}
+
 	// write all running AWS drivers to serial port.
 	if driverInfo, err = getAWSDriverInfo(log); err == nil {
 		for _, di := range driverInfo {
@@ -324,6 +339,17 @@ func getAWSPvPackageInfo(log log.T) (pvPackageInfo model.PvPackageInfo, err erro
 	return
 }
 
+// getAWSNitroEnclavesPackage queries AwsNitroEnclaves information from registry key.
+func getAWSNitroEnclavesPackageInfo(log log.T) (NitroEnclavesPackageInfo model.NitroEnclavesPackageInfo, err error) {
+
+	// this queries the registry for AWS Nitro Enclaves Package version
+	properties := []string{NitroEnclavesName, NitroEnclavesVersionProperty}
+	if err = runPowershell(&NitroEnclavesPackageInfo, getNitroEnclavesPackageVersionCmd, properties, false); err != nil {
+		log.Debugf("Error occurred while querying Version for AWSNitroEnclavesPackage: %v", err.Error())
+	}
+	return
+}
+
 // getAWSDriverInfo queries driver information from instance using powershell.
 // because Nano server doesn't support Win32_PnpSignedDriver.
 func getAWSDriverInfo(log log.T) (driverInfo []model.DriverInfo, err error) {
@@ -469,7 +495,7 @@ func runPowershell(jsonObj interface{}, command string, properties []string, exp
 	args = append(args, "| ConvertTo-Json -Depth 3")
 
 	// execute powershell with arguments in cmd.
-	cmdOut, err = cmdExec.ExecuteCommand("powershell", args...)
+	cmdOut, err = cmdExec.ExecuteCommand(appconfig.PowerShellPluginCommandName, args...)
 	if err != nil {
 		err = errors.New(fmt.Sprintf("Error while running powershell %v: %v", args, err.Error()))
 		return
