@@ -16,26 +16,31 @@
 package processor
 
 import (
+	"bytes"
 	"fmt"
 	"math/rand"
 	"os"
 	"path/filepath"
+	"strings"
 	"sync"
 	"time"
 
+	"github.com/aws/amazon-ssm-agent/common/utility"
+
 	"github.com/aws/amazon-ssm-agent/agent/appconfig"
 	"github.com/aws/amazon-ssm-agent/agent/context"
 	"github.com/aws/amazon-ssm-agent/agent/contracts"
 	"github.com/aws/amazon-ssm-agent/agent/fileutil"
 	"github.com/aws/amazon-ssm-agent/agent/fileutil/artifact"
 	"github.com/aws/amazon-ssm-agent/agent/log"
+	"github.com/aws/amazon-ssm-agent/agent/platform"
 	testerPkg "github.com/aws/amazon-ssm-agent/agent/update/tester"
-	testerCommon "github.com/aws/amazon-ssm-agent/agent/update/tester/common"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateconstants"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateprecondition"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updates3util"
+	"github.com/aws/amazon-ssm-agent/agent/version"
 	"github.com/aws/amazon-ssm-agent/agent/versionutil"
 )
 
@@ -43,8 +48,16 @@ var minimumSupportedVersions map[string]string
 var once sync.Once
 
 var (
-	downloadArtifact = artifact.Download
-	uncompress       = fileutil.Uncompress
+	downloadArtifact           = artifact.Download
+	uncompress                 = fileutil.Uncompress
+	getInstalledVersionRef     = getInstalledVersions
+	getDirectoryNames          = fileutil.GetDirectoryNames
+	deleteDirectory            = os.RemoveAll
+	getStableManifestURL       = updateutil.GetStableURLFromManifestURL
+	getFileNamesLaterThan      = fileutil.GetFileNamesUnsortedLaterThan
+	moveFile                   = fileutil.MoveFile
+	waitForCloudInit           = utility.WaitForCloudInit
+	isWindowsServer2025OrLater = platform.IsWindowsServer2025OrLater
 )
 
 const (
@@ -52,18 +65,19 @@ const (
 	defaultStderrFileName      = "stderr"
 	defaultSSMAgentName        = "amazon-ssm-agent"
 	defaultSelfUpdateMessageID = "aws.ssm.self-update-agent.i-instanceid"
+	installationDirectory      = "installationDir"
+	cloudInitWaitSeconds       = 600
+	agentVersionPriorWin2025   = "3.3.1230.0"
 )
 
 // NewUpdater creates an instance of Updater and other services it requires
-func NewUpdater(context context.T, info updateinfo.T) *Updater {
+func NewUpdater(context context.T, info updateinfo.T, updateUtilRef *updateutil.Utility) *Updater {
 	updater := &Updater{
 		mgr: &updateManager{
 			Context: context,
 			Info:    info,
-			util: &updateutil.Utility{
-				Context: context,
-			},
-			S3util: updates3util.New(context),
+			util:    updateUtilRef,
+			S3util:  updates3util.New(context),
 			svc: &svcManager{
 				context: context,
 			},
@@ -83,7 +97,7 @@ func NewUpdater(context context.T, info updateinfo.T) *Updater {
 			uninstall:           uninstallAgent,
 			install:             installAgent,
 			download:            downloadAndUnzipArtifact,
-			clean:               cleanUninstalledVersions,
+			clean:               cleanAgentArtifacts,
 			runTests:            testerPkg.StartTests,
 			finalize:            finalizeUpdateAndSendReply,
 		},
@@ -151,21 +165,31 @@ func (u *Updater) Failed(updateDetail *UpdateDetail, log log.T, code updateconst
 
 // validateUpdateVersion validates target version number base on the current platform
 // to avoid accidentally downgrade agent to the earlier version that doesn't support current platform
-func validateUpdateVersion(log log.T, detail *UpdateDetail, info updateinfo.T) (err error) {
-	compareResult := 0
+func validateUpdateVersion(log log.T, detail *UpdateDetail, info updateinfo.T) error {
 	minimumVersions := getMinimumVSupportedVersions()
 
 	// check if current platform has minimum supported version
 	if val, ok := (*minimumVersions)[info.GetPlatform()]; ok {
 		// compare current agent version with minimum supported version
-		if compareResult, err = versionutil.VersionCompare(detail.TargetVersion, val); err != nil {
+		if compareResult, err := versionutil.VersionCompare(detail.TargetVersion, val); err != nil {
 			return err
-		}
-		if compareResult < 0 {
+		} else if compareResult < 0 {
 			return fmt.Errorf("agent version %v is unsupported on current platform", detail.TargetVersion)
 		}
 	}
 
+	if windows2025OrLater, err := isWindowsServer2025OrLater(info.GetPlatformVersion(), log); err != nil {
+		return err
+	} else if windows2025OrLater {
+		// target agent version has to be greater than the lastest agent version that didn't have support for Win 2025,
+		// in order to work with it or with any other Win released after
+		if compareResult, err := versionutil.VersionCompare(detail.TargetVersion, agentVersionPriorWin2025); err != nil {
+			return err
+		} else if compareResult <= 0 {
+			return fmt.Errorf("agent version %s is unsupported on current platform", detail.TargetVersion)
+		}
+	}
+
 	return nil
 }
 
@@ -190,6 +214,15 @@ func validateInactiveVersion(context context.T, info updateinfo.T, detail *Updat
 	return nil
 }
 
+func validateTargetVersionCompatible(detail *UpdateDetail) (err error) {
+	if detail.UpstreamServiceName == string(contracts.MessageGatewayService) {
+		if versionutil.Compare(detail.TargetVersion, updateconstants.DowngradeThroughMGSMinVersion, true) < 0 {
+			return fmt.Errorf("before downgrading to %s, first downgrade to any version from 3.1.821.0 to 3.2.923.0", detail.TargetVersion)
+		}
+	}
+	return nil
+}
+
 // getMinimumVSupportedVersions returns a map of minimum supported version and it's platform
 func getMinimumVSupportedVersions() (versions *map[string]string) {
 	once.Do(func() {
@@ -214,9 +247,10 @@ func initManifest(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (er
 	}
 
 	log.Infof("Initiating download manifest %v", updateDetail.ManifestURL)
-	err = mgr.S3util.DownloadManifest(updateDetail.Manifest, updateDetail.ManifestURL)
-	if err != nil {
-		return mgr.failed(updateDetail, log, updateconstants.ErrorDownloadManifest, fmt.Sprintf("Failed to download manifest: %v", err), true)
+	downloadErr := mgr.S3util.DownloadManifest(updateDetail.Manifest, updateDetail.ManifestURL)
+	if downloadErr != nil && downloadErr.Error != nil {
+		errorCode := updateutil.ConvertToUpdateErrorCode(string(updateconstants.ErrorDownloadManifest), "_", downloadErr.ErrorCode)
+		return mgr.failed(updateDetail, log, errorCode, fmt.Sprintf("Failed to download manifest: %v", downloadErr.Error.Error()), true)
 	}
 
 	return mgr.initSelfUpdate(mgr, log, updateDetail)
@@ -274,6 +308,20 @@ func determineTarget(mgr *updateManager, logger log.T, updateDetail *UpdateDetai
 		if err != nil {
 			return mgr.failed(updateDetail, logger, updateconstants.ErrorGetLatestActiveVersionManifest, fmt.Sprintf("Failed to get latest active version from manifest: %v", err), true)
 		}
+	} else if strings.ToLower(updateDetail.TargetVersion) == "stable" {
+		var stableVersionUrl string
+		updateDetail.TargetResolver = updateconstants.TargetVersionStable
+		logger.Info("TargetVersion is 'stable', attempting to get the stable version from s3")
+		stableVersionUrl, err = getStableManifestURL(updateDetail.ManifestURL, mgr.Context.Identity())
+		if err != nil {
+			// Should never happen because manifest has already been downloaded using the manifest url at this point
+			return mgr.failed(updateDetail, logger, updateconstants.ErrorGetStableVersionS3, fmt.Sprintf("Failed to generate stable version from manifest url: %v", err), true)
+		}
+
+		updateDetail.TargetVersion, err = mgr.S3util.GetStableVersion(stableVersionUrl)
+		if err != nil {
+			return mgr.failed(updateDetail, logger, updateconstants.ErrorGetStableVersionS3, fmt.Sprintf("Failed to get stable version form s3: %v", err), true)
+		}
 	} else {
 		updateDetail.TargetResolver = updateconstants.TargetVersionCustomerDefined
 	}
@@ -326,7 +374,7 @@ func validateUpdateParam(mgr *updateManager, logger log.T, updateDetail *UpdateD
 
 		if !updateDetail.AllowDowngrade {
 			logger.Warnf("Downgrade is not enabled, please enable downgrade to perform this update")
-			return mgr.failed(updateDetail, logger, updateconstants.ErrorAttemptToDowngrade, fmt.Sprintf("Updating %v to an older version, please enable allow downgrade to proceed", updateDetail.TargetVersion), true)
+			return mgr.failed(updateDetail, logger, updateconstants.ErrorAttemptToDowngrade, fmt.Sprintf("Updating %v to an older version %v, please enable allow downgrade to proceed", updateDetail.SourceVersion, updateDetail.TargetVersion), true)
 		}
 	}
 
@@ -349,6 +397,11 @@ func validateUpdateParam(mgr *updateManager, logger log.T, updateDetail *UpdateD
 		return mgr.inactive(updateDetail, logger, updateconstants.WarnInactiveVersion)
 	}
 
+	// Validate target version is compatible with update coming from upstream messaging service
+	if err = validateTargetVersionCompatible(updateDetail); err != nil {
+		return mgr.failed(updateDetail, logger, updateconstants.ErrorIncompatibleTargetVersion, err.Error(), true)
+	}
+
 	// Checking target version update preconditions
 	for _, condition := range mgr.preconditions {
 		logger.Infof("Checking update precondition %s", condition.GetPreconditionName())
@@ -421,10 +474,8 @@ func downloadPackages(mgr *updateManager, log log.T, updateDetail *UpdateDetail)
 		updateDetail.PackageName,
 		updateDetail.TargetVersion)
 
-	// Update state to Staged
-	if err = mgr.inProgress(updateDetail, log, Staged); err != nil {
-		return err
-	}
+	updateDetail.State = Staged
+	updateDetail.Result = contracts.ResultStatusInProgress
 
 	// Process update
 	return mgr.update(mgr, log, updateDetail)
@@ -432,8 +483,12 @@ func downloadPackages(mgr *updateManager, log log.T, updateDetail *UpdateDetail)
 
 // proceedUpdate starts update process
 func proceedUpdate(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (err error) {
+	if err := waitForCloudInit(log, cloudInitWaitSeconds); err != nil {
+		log.Warnf("error waiting for cloud-init: %v", err)
+	}
+
 	log.Infof(
-		"Attemping to upgrade from %v to %v",
+		"Attempting to upgrade from %v to %v",
 		updateDetail.SourceVersion,
 		updateDetail.TargetVersion)
 
@@ -452,10 +507,13 @@ func proceedUpdate(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (e
 			return mgr.failed(updateDetail, log, updateconstants.ErrorUninstallFailed, message, true)
 		}
 	}
-	preInstallTestTimeoutSeconds := 7
-	if testResult := mgr.runTests(mgr.Context, testerCommon.PreInstallTest, preInstallTestTimeoutSeconds); testResult != "" {
-		mgr.reportTestFailure(updateDetail, log, testResult)
+
+	mgr.runTests(mgr.Context, mgr.reportTestResultGenerator(updateDetail, log))
+
+	if err = mgr.util.UpdateInstallDelayer(mgr.Context, updateDetail.UpdateRoot); err != nil {
+		log.Errorf("error while executing install delayer %v", err)
 	}
+
 	if exitCode, err := mgr.install(mgr, log, updateDetail.TargetVersion, updateDetail); err != nil {
 		// Install target failed with err
 		// log the error and initiating rollback to the source version
@@ -468,7 +526,9 @@ func proceedUpdate(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (e
 		if exitCode == updateconstants.ExitCodeUnsupportedPlatform {
 			return mgr.failed(updateDetail, log, updateconstants.ErrorUnsupportedServiceManager, message, true)
 		}
-
+		if exitCode == updateconstants.ExitCodeUpdateFailedDueToSnapd {
+			return mgr.failed(updateDetail, log, updateconstants.ErrorInstallFailureDueToSnapd, message, true)
+		}
 		updateDetail.AppendInfo(
 			log,
 			"Initiating rollback %v to %v",
@@ -483,11 +543,8 @@ func proceedUpdate(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (e
 		return mgr.rollback(mgr, log, updateDetail)
 	}
 
-	// Update state to installed to indicate there is no error occur during installation
-	// Updater has installed the new version and started the verify process
-	if err = mgr.inProgress(updateDetail, log, Installed); err != nil {
-		return err
-	}
+	updateDetail.State = Installed
+	updateDetail.Result = contracts.ResultStatusInProgress
 
 	// verify target version
 	return mgr.verify(mgr, log, updateDetail, false)
@@ -502,7 +559,7 @@ func verifyInstallation(mgr *updateManager, log log.T, updateDetail *UpdateDetai
 	if isRollback {
 		version = updateDetail.SourceVersion
 	}
-	log.Infof("Initiating update health check")
+	log.Info("Initiating Agent service check")
 	if isRunning, err = mgr.util.WaitForServiceToStart(log, mgr.Info, version); err != nil || !isRunning {
 		if !isRollback {
 			message := updateutil.BuildMessage(err,
@@ -534,8 +591,17 @@ func verifyInstallation(mgr *updateManager, log log.T, updateDetail *UpdateDetai
 		return mgr.failed(updateDetail, log, updateconstants.ErrorCannotStartService, message, false)
 	}
 
-	log.Infof("%v is running", updateDetail.PackageName)
 	if !isRollback {
+		// initiate rollback when agent installation is not complete
+		if versionInstalledErrCode := mgr.util.VerifyInstalledVersion(log, version); versionInstalledErrCode != "" {
+			message := updateutil.BuildMessage(nil,
+				"failed to identify installed target agent version: %v",
+				updateDetail.TargetVersion)
+			updateDetail.AppendError(log, message)
+			// we will receive only 1 error code - ErrorInstTargetVersionNotFoundViaReg
+			return mgr.failed(updateDetail, log, versionInstalledErrCode, message, false)
+		}
+		log.Infof("%v is running", updateDetail.PackageName)
 		return mgr.succeeded(updateDetail, log)
 	}
 
@@ -604,16 +670,19 @@ func uninstallAgent(mgr *updateManager, log log.T, version string, updateDetail
 	uninstallRetryCount := 3
 	uninstallRetryDelay := 1000     // 1 second
 	uninstallRetryDelayBase := 2000 // 2 seconds
+	input := &updateutil.CommandExecutionSettings{
+		Log:         log,
+		Cmd:         strings.Fields(uninstallPath),
+		WorkingDir:  workDir,
+		UpdaterRoot: updateDetail.UpdateRoot,
+		StdOut:      updateDetail.StdoutFileName,
+		StdErr:      updateDetail.StderrFileName,
+		IsAsync:     false,
+		Env:         getCommandEnv(log, updateDetail.UpdateRoot),
+	}
 	// Uninstall version - TODO - move the retry logic to ExeCommand while cleaning that function
 	for retryCounter := 1; retryCounter <= uninstallRetryCount; retryCounter++ {
-		_, exitCode, err = mgr.util.ExeCommand(
-			log,
-			uninstallPath,
-			workDir,
-			updateDetail.UpdateRoot,
-			updateDetail.StdoutFileName,
-			updateDetail.StderrFileName,
-			false)
+		_, exitCode, err = mgr.util.ExeCommand(input)
 		if err == nil {
 			break
 		}
@@ -649,15 +718,33 @@ func installAgent(mgr *updateManager, log log.T, version string, updateDetail *U
 	if updateDetail.State == Staged {
 		installRetryCount = 4 // this value is taken because previous updater version had total 4 retries (2 target install + 2 rollback install)
 	}
+	defaultTimeOut := mgr.util.GetExecutionTimeOut()
+
+	input := &updateutil.CommandExecutionSettings{
+		Log:         log,
+		Cmd:         strings.Fields(installerPath),
+		WorkingDir:  workDir,
+		UpdaterRoot: updateDetail.UpdateRoot,
+		StdOut:      updateDetail.StdoutFileName,
+		StdErr:      updateDetail.StderrFileName,
+		IsAsync:     false,
+		Env:         getCommandEnv(log, updateDetail.UpdateRoot),
+	}
 	for retryCounter := 1; retryCounter <= installRetryCount; retryCounter++ {
-		_, exitCode, err = mgr.util.ExeCommand(
-			log,
-			installerPath,
-			workDir,
-			updateDetail.UpdateRoot,
-			updateDetail.StdoutFileName,
-			updateDetail.StderrFileName,
-			false)
+		updateExecutionTimeoutIfNeeded(retryCounter, defaultTimeOut, mgr.util)
+		if retryCounter == installRetryCount && strings.Contains(mgr.Info.GetInstallScriptName(), "snap") {
+			log.Info("execute command and fetch error output for agent install using snap")
+			var errBytes *bytes.Buffer
+			_, exitCode, _, errBytes, err = mgr.util.ExecCommandWithOutput(input)
+			if err != nil && errBytes != nil && errBytes.Len() != 0 {
+				if strings.Contains(errBytes.String(), "snap \"amazon-ssm-agent\" has running apps") {
+					log.Errorf("command failure for agent installed using snap: %v", err)
+					return updateconstants.ExitCodeUpdateFailedDueToSnapd, err
+				}
+			}
+		} else {
+			_, exitCode, err = mgr.util.ExeCommand(input)
+		}
 		if err == nil {
 			break
 		}
@@ -693,45 +780,82 @@ func getNextBackOff(retryCounter int) (backOff time.Duration) {
 	return time.Duration((backOffSeconds*1000)+rand.Intn(randomDelay)) * time.Millisecond
 }
 
-// cleanUninstalledVersions deletes leftover files from previously installed versions in update folder
-func cleanUninstalledVersions(mgr *updateManager, log log.T, updateDetail *UpdateDetail) (err error) {
-	log.Infof("Initiating cleanup of other versions.")
+// getInstalledVersions returns the version which needs to be retained after deletion
+// returns empty for other folders except amazon-ssm-agent, amazon-ssm-agent-updater download folders
+func getInstalledVersions(updateDetail *UpdateDetail, path string) string {
+	var validVersion string
+	updaterBasePath := filepath.Base(path)
+	if updaterBasePath == updateconstants.UpdateAmazonSSMAgentDir {
+		switch updateDetail.Result {
+		case contracts.ResultStatusSuccess:
+			validVersion = updateDetail.TargetVersion
+		case contracts.ResultStatusFailed:
+			validVersion = updateDetail.SourceVersion
+		}
+	} else if updaterBasePath == updateconstants.UpdateAmazonSSMAgentUpdaterDir {
+		validVersion = version.Version
+	}
+	return validVersion
+}
+
+// cleanAgentArtifacts deletes leftover files from update folders (amazon-ssm-agent, amazon-ssm-agent-updater and downloads folder)
+func cleanAgentArtifacts(log log.T, updateDetail *UpdateDetail) {
+	moveCleanInstallationDir(log, updateDetail)
+	_ = cleanAgentUpdaterDir(log, updateDetail)
+	cleanUpdateDownloadDir(log)
+}
+
+// cleanAgentUpdaterDir deletes files from update folders (amazon-ssm-agent and amazon-ssm-agent-updater folder)
+func cleanAgentUpdaterDir(log log.T, updateDetail *UpdateDetail) (rmVersions map[string][]string) {
+	log.Infof("initiating cleanup of other versions in amazon-ssm-agent and amazon-ssm-agent-updater folder")
 	var installedVersion string
+	var combinedErrors bytes.Buffer
+	removedVersions := make(map[string][]string, 2)
+	ssmAgentDownloadDir := filepath.Join(appconfig.UpdaterArtifactsRoot, updateconstants.UpdateAmazonSSMAgentDir)
+	ssmAgentUpdaterDir := filepath.Join(appconfig.UpdaterArtifactsRoot, updateconstants.UpdateAmazonSSMAgentUpdaterDir)
 
-	switch updateDetail.Result {
-	case contracts.ResultStatusSuccess:
-		installedVersion = updateDetail.TargetVersion
-	case contracts.ResultStatusFailed:
-		installedVersion = updateDetail.SourceVersion
-	}
-
-	path := appconfig.UpdaterArtifactsRoot + updateconstants.UpdateAmazonSSMAgentDir
-	directoryNames, err := fileutil.GetDirectoryNames(path)
-
-	if err != nil {
-		return err
-	}
-
-	removedVersions := ""
-	combinedErrors := fmt.Errorf("")
-	for _, directoryName := range directoryNames {
-		if directoryName == installedVersion {
-			continue
-		} else if err = fileutil.DeleteDirectory(path + directoryName); err != nil {
-			combinedErrors = fmt.Errorf(combinedErrors.Error() + err.Error() + "\n")
+	artifactDirPaths := []string{ssmAgentDownloadDir, ssmAgentUpdaterDir}
+	for _, artifactDirPath := range artifactDirPaths {
+		log.Infof("removing artifacts in the folder: %s", artifactDirPath)
+		installedVersion = getInstalledVersionRef(updateDetail, artifactDirPath)
+		artifactNames, dirErr := getDirectoryNames(artifactDirPath)
+		if dirErr == nil {
+			for _, artifactName := range artifactNames {
+				artifactDir := filepath.Join(artifactDirPath, artifactName)
+				if artifactName == installedVersion {
+					// we skip installed/running versions to avoid race conditions while deleting
+					continue
+				} else if err := deleteDirectory(artifactDir); err != nil {
+					combinedErrors.WriteString(err.Error())
+					combinedErrors.WriteString("\n")
+				} else {
+					removedVersions[artifactDirPath] = append(removedVersions[artifactDirPath], artifactName)
+				}
+			}
+			log.Infof("removed files and folders: %v", strings.Join(removedVersions[artifactDirPath], ", "))
 		} else {
-			removedVersions += directoryName + "\n"
+			log.Warnf("failed to list directory names: %v", dirErr)
 		}
 	}
-
-	log.Infof("Installed version: %v", installedVersion)
-	log.Infof("Removed versions: %v", removedVersions)
-
-	if combinedErrors.Error() != "" {
-		return combinedErrors
+	if combinedErrors.String() != "" {
+		log.Warnf("combined errors while deleting files in updater and agent download folders: %v", combinedErrors.String())
 	}
+	return removedVersions
+}
 
-	return nil
+// cleanUpdateDownloadDir deletes files from update download folders (update download folder)
+func cleanUpdateDownloadDir(log log.T) {
+	log.Infof("initiating cleanup of files in update download folder")
+	updateDownloadDir := filepath.Join(appconfig.DownloadRoot, "update")
+	legacyUpdateDownloadDir := filepath.Join(appconfig.LegacyUpdateDownloadFolder, "update")
+	if fileutil.Exists(legacyUpdateDownloadDir) {
+		if err := deleteDirectory(legacyUpdateDownloadDir); err != nil {
+			log.Warnf("error while deleting the update legacy download folder: %v", err)
+		}
+	}
+	if err := deleteDirectory(updateDownloadDir); err != nil {
+		log.Warnf("error while deleting the update download folder: %v", err)
+	}
 }
 
 // downloadAndUnzipArtifact downloads installation package and unzips it
