@@ -15,12 +15,14 @@ package updateprecondition
 
 import (
 	"github.com/aws/amazon-ssm-agent/agent/context"
+	bottlerocketprecondition "github.com/aws/amazon-ssm-agent/agent/updateutil/updateprecondition/bottlerocket"
 	staticpieprecondition "github.com/aws/amazon-ssm-agent/agent/updateutil/updateprecondition/staticpie"
 )
 
 // GetPreconditions returns a list of all update preconditions that the updater should check
 func GetPreconditions(context context.T) []T {
 	return []T{
+		bottlerocketprecondition.New(context),
 		staticpieprecondition.New(context),
 	}
 }
