@@ -16,6 +16,7 @@ package updateutil
 
 import (
 	"bufio"
+	"bytes"
 	"errors"
 	"fmt"
 	"io"
@@ -31,24 +32,46 @@ import (
 
 	"github.com/aws/amazon-ssm-agent/agent/appconfig"
 	"github.com/aws/amazon-ssm-agent/agent/context"
+	"github.com/aws/amazon-ssm-agent/agent/contracts"
 	"github.com/aws/amazon-ssm-agent/agent/fileutil"
 	"github.com/aws/amazon-ssm-agent/agent/log"
+	"github.com/aws/amazon-ssm-agent/agent/s3util"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateconstants"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo"
 	"github.com/aws/amazon-ssm-agent/agent/versionutil"
+	"github.com/aws/amazon-ssm-agent/common/identity"
+	identity2 "github.com/aws/amazon-ssm-agent/common/identity/identity"
 	"github.com/aws/amazon-ssm-agent/core/executor"
 	"github.com/aws/amazon-ssm-agent/core/workerprovider/longrunningprovider/model"
 )
 
+type CommandExecutionSettings struct {
+	Log         log.T
+	Cmd         []string
+	WorkingDir  string
+	UpdaterRoot string
+	StdOut      string
+	StdErr      string
+	IsAsync     bool
+	Env         []string
+}
+
 // T represents the interface for Update utility
 type T interface {
 	CreateUpdateDownloadFolder() (folder string, err error)
-	ExeCommand(log log.T, cmd string, workingDir string, updaterRoot string, stdOut string, stdErr string, isAsync bool) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error)
+	ExeCommand(input *CommandExecutionSettings) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error)
+	ExeCommandWithSlice(input *CommandExecutionSettings) (pid int, exitCode updateconstants.UpdateScriptExitCode, err error)
+	ExecCommandWithOutput(input *CommandExecutionSettings) (pId int, updExitCode updateconstants.UpdateScriptExitCode, stdoutBytes *bytes.Buffer, errorBytes *bytes.Buffer, cmdErr error)
 	IsServiceRunning(log log.T, i updateinfo.T) (result bool, err error)
 	IsWorkerRunning(log log.T) (result bool, err error)
 	WaitForServiceToStart(log log.T, i updateinfo.T, targetVersion string) (result bool, err error)
 	SaveUpdatePluginResult(log log.T, updaterRoot string, updateResult *UpdatePluginResult) (err error)
 	IsDiskSpaceSufficientForUpdate(log log.T) (bool, error)
+	UpdateInstallDelayer(ctx context.T, updateRoot string) error
+	LoadUpdateDocumentState(ctx context.T, commandId string) error
+	VerifyInstalledVersion(log log.T, targetVersion string) updateconstants.ErrorCode
+	UpdateExecutionTimeOut(int)
+	GetExecutionTimeOut() int
 }
 
 // Utility implements interface T
@@ -56,16 +79,33 @@ type Utility struct {
 	Context                               context.T
 	CustomUpdateExecutionTimeoutInSeconds int
 	ProcessExecutor                       executor.IExecutor
+	UpdateDocState                        contracts.DocumentState
 }
 
 var getDiskSpaceInfo = fileutil.GetDiskSpaceInfo
+var getRemoteProvider = identity2.GetRemoteProvider
 var mkDirAll = os.MkdirAll
 var openFile = os.OpenFile
 var execCommand = exec.Command
 var cmdStart = (*exec.Cmd).Start
 var cmdOutput = (*exec.Cmd).Output
 
-// CreateInstanceContext create instance related information such as region, platform and arch
+const (
+	stateJson = "updatestate.json"
+)
+
+func NewUpdaterUtilWithLoadedDocContent(ctx context.T, commandId string) *Utility {
+	updateUtil := &Utility{
+		Context: ctx,
+	}
+	err := updateUtil.LoadUpdateDocumentState(ctx, commandId)
+	if err != nil {
+		ctx.Log().Warnf("Could not load doc content for commandID /%v/: %v", commandId, err)
+	}
+	return updateUtil
+}
+
+// CreateInstanceInfo create instance related information such as region, platform and arch
 func (util *Utility) CreateInstanceInfo(log log.T) (context updateinfo.T, err error) {
 	return nil, nil
 }
@@ -80,23 +120,98 @@ func (util *Utility) CreateUpdateDownloadFolder() (folder string, err error) {
 	return root, nil
 }
 
-// ExeCommand executes shell command
-func (util *Utility) ExeCommand(
-	log log.T,
-	cmd string,
-	workingDir string,
-	outputRoot string,
-	stdOut string,
-	stdErr string,
-	isAsync bool) (int, updateconstants.UpdateScriptExitCode, error) { // pid, exitCode, error
+// UpdateExecutionTimeOut updates the command execution timeout
+func (util *Utility) UpdateExecutionTimeOut(timeout int) {
+	util.CustomUpdateExecutionTimeoutInSeconds = timeout
+}
 
-	parts := strings.Fields(cmd)
+// GetExecutionTimeOut returns the command execution timeout
+func (util *Utility) GetExecutionTimeOut() int {
+	return util.CustomUpdateExecutionTimeoutInSeconds
+}
+
+// ExecCommandWithOutput executes shell command and returns output and error of command execution
+func (util *Utility) ExecCommandWithOutput(input *CommandExecutionSettings) (pId int, updExitCode updateconstants.UpdateScriptExitCode, stdoutBytes *bytes.Buffer, errorBytes *bytes.Buffer, cmdErr error) {
+	log := input.Log
+	parts := input.Cmd
 	pid := -1
 	var updateExitCode updateconstants.UpdateScriptExitCode = -1
+	tempCmd := setPlatformSpecificCommand(parts)
+	command := execCommand(tempCmd[0], tempCmd[1:]...)
+	command.Dir = input.WorkingDir
 
-	if isAsync {
-		command := execCommand(parts[0], parts[1:]...)
-		command.Dir = workingDir
+	// Update command env if set by the caller
+	// If not set, command passes the os.Environ() by default
+	if input.Env != nil && len(input.Env) > 0 {
+		command.Env = input.Env
+	}
+
+	stdoutWriter, stderrWriter, err := setExeOutErr(input.UpdaterRoot, input.StdOut, input.StdErr)
+	if err != nil {
+		return pid, updateExitCode, nil, nil, err
+	}
+	defer stdoutWriter.Close()
+	defer stderrWriter.Close()
+	var errBytes, stdOutBytes bytes.Buffer
+	command.Stdout = &stdOutBytes
+	command.Stderr = &errBytes
+
+	err = cmdStart(command)
+	if err != nil {
+		return pid, updateExitCode, &stdOutBytes, &errBytes, err
+	}
+
+	pid = GetCommandPid(command)
+
+	var timeout = updateconstants.DefaultUpdateExecutionTimeoutInSeconds
+	if util.CustomUpdateExecutionTimeoutInSeconds != 0 {
+		timeout = util.CustomUpdateExecutionTimeoutInSeconds
+	}
+	timer := time.NewTimer(time.Duration(timeout) * time.Second)
+	go killProcessOnTimeout(log, command, timer)
+	err = command.Wait()
+	if errBytes.Len() != 0 {
+		stderrWriter.Write(errBytes.Bytes())
+	}
+	if stdOutBytes.Len() != 0 {
+		stdoutWriter.Write(stdOutBytes.Bytes())
+	}
+	timedOut := !timer.Stop()
+	if err != nil {
+		log.Debugf("command returned error %v", err)
+		if exitErr, ok := err.(*exec.ExitError); ok {
+			// The program has exited with an exit code != 0
+			if status, ok := exitErr.Sys().(syscall.WaitStatus); ok {
+				exitCode := status.ExitStatus()
+				if exitCode == -1 && timedOut {
+					// set appropriate exit code based on cancel or timeout
+					exitCode = appconfig.CommandStoppedPreemptivelyExitCode
+					log.Infof("The execution of command was timedout.")
+				}
+				updateExitCode = updateconstants.UpdateScriptExitCode(exitCode)
+				err = fmt.Errorf("The execution of command returned Exit Status: %d \n %v", exitCode, err.Error())
+			}
+		}
+		return pid, updateExitCode, &stdOutBytes, &errBytes, err
+	}
+	return pid, updateExitCode, &stdOutBytes, nil, nil
+}
+
+// ExeCommand executes shell command
+func (util *Utility) ExeCommand(input *CommandExecutionSettings) (int, updateconstants.UpdateScriptExitCode, error) { // pid, exitCode, error
+	return util.ExeCommandWithSlice(input)
+}
+
+// ExeCommandWithSlice executes shell command
+func (util *Utility) ExeCommandWithSlice(input *CommandExecutionSettings) (int, updateconstants.UpdateScriptExitCode, error) { // pid, exitCode, error
+	log := input.Log
+	pid := -1
+	var updateExitCode updateconstants.UpdateScriptExitCode = -1
+	cmd := input.Cmd
+	if input.IsAsync {
+		command := execCommand(cmd[0], cmd[1:]...)
+		command.Dir = input.WorkingDir
+		util.setCommandEnvironmentVariables(command, input.Env)
 		prepareProcess(command)
 		// Start command asynchronously
 		err := cmdStart(command)
@@ -105,10 +220,16 @@ func (util *Utility) ExeCommand(
 		}
 		pid = GetCommandPid(command)
 	} else {
-		tempCmd := setPlatformSpecificCommand(parts)
+		tempCmd := setPlatformSpecificCommand(cmd)
 		command := execCommand(tempCmd[0], tempCmd[1:]...)
-		command.Dir = workingDir
-		stdoutWriter, stderrWriter, err := setExeOutErr(outputRoot, stdOut, stdErr)
+		command.Dir = input.WorkingDir
+
+		// Update command env if set by the caller
+		// If not set, command passes the os.Environ() by default
+		if input.Env != nil && len(input.Env) > 0 {
+			command.Env = input.Env
+		}
+		stdoutWriter, stderrWriter, err := setExeOutErr(input.UpdaterRoot, input.StdOut, input.StdErr)
 		if err != nil {
 			return pid, updateExitCode, err
 		}
@@ -177,6 +298,7 @@ func (util *Utility) ExeCommandOutput(
 
 	command := execCommand(tempCmd[0], tempCmd[1:]...)
 	command.Dir = workingDir
+	util.setCommandEnvironmentVariables(command, nil)
 	stdoutWriter, stderrWriter, exeErr := setExeOutErr(outputRoot, stdOutFileName, stdErrFileName)
 	if exeErr != nil {
 		return output, exeErr
@@ -190,13 +312,14 @@ func (util *Utility) ExeCommandOutput(
 	// Run the command and return its output
 	var out []byte
 	out, err = cmdOutput(command)
+	output = string(out)
 	// Write the returned output so that we can upload it if needed
 	stdoutWriter.Write(out)
 	if err != nil {
 		return
 	}
 
-	return string(out), err
+	return output, err
 }
 
 // TODO move to commandUtil
@@ -221,20 +344,21 @@ func (util *Utility) NewExeCommandOutput(
 
 	command := execCommand(tempCmd[0], tempCmd[1:]...)
 	command.Dir = workingDir
-
+	util.setCommandEnvironmentVariables(command, nil)
 	// Don't set command.Stdout - we're going to return it instead of writing it
 	command.Stderr = stderrWriter
 
 	// Run the command and return its output
 	var out []byte
 	out, err = cmdOutput(command)
+	output = string(out)
 	// Write the returned output so that we can upload it if needed
 	stdoutWriter.Write(out)
 	if err != nil {
 		return
 	}
 
-	return string(out), err
+	return output, err
 }
 
 // IsServiceRunning returns is service running
@@ -331,6 +455,12 @@ func (util *Utility) IsWorkerRunning(log log.T) (result bool, err error) {
 	return false, nil
 }
 
+// VerifyInstalledVersion verifies whether the expected version is installed on Windows instance.
+// For linux, this function always return true
+func (util *Utility) VerifyInstalledVersion(log log.T, targetVersion string) updateconstants.ErrorCode {
+	return verifyVersion(log, targetVersion)
+}
+
 // WaitForServiceToStart wait for service to start and returns is service started
 func (util *Utility) WaitForServiceToStart(log log.T, i updateinfo.T, targetVersion string) (result bool, svcRunningErr error) {
 	const (
@@ -614,6 +744,15 @@ func parseVersion(version string) (uint64, uint64, uint64, uint64, error) {
 	return major, minor, build, patch, nil
 }
 
+// ConvertToUpdateErrorCode joins the error string array passed and converts to update error code
+func ConvertToUpdateErrorCode(errorCodes ...string) updateconstants.ErrorCode {
+	var updateErrorCode bytes.Buffer
+	for _, errCode := range errorCodes {
+		updateErrorCode.WriteString(errCode)
+	}
+	return updateconstants.ErrorCode(updateErrorCode.String())
+}
+
 // GetManifestURLFromSourceUrl parses source url passed to the updater and generates the url for manifest
 func GetManifestURLFromSourceUrl(sourceURL string) (string, error) {
 	u, err := url.Parse(sourceURL)
@@ -638,12 +777,84 @@ func GetManifestURLFromSourceUrl(sourceURL string) (string, error) {
 	return u.String(), nil
 }
 
+func GetStableURLFromManifestURL(manifestURL string, identity identity.IAgentIdentity) (string, error) {
+	if !strings.HasSuffix(manifestURL, "/"+updateconstants.ManifestFile) {
+		return "", fmt.Errorf("unexpected manifest url does not end with manifest file: %s", manifestURL)
+	}
+	stableVersionPath := "stable/VERSION"
+	region, err := identity.Region()
+	if err != nil {
+		return "", err
+	}
+
+	stableVersionURL := strings.TrimRight(manifestURL, updateconstants.ManifestFile) + stableVersionPath
+	stableVersionURL = strings.Replace(stableVersionURL, updateconstants.RegionHolder, region, -1)
+	return stableVersionURL, nil
+}
+
+// ResolveAgentReleaseBucketURL makes best effort to generate an url for the ssm agent bucket
+func ResolveAgentReleaseBucketURL(region string, identity identity.IAgentIdentity) string {
+	s3Url := ""
+	if dynamicS3Endpoint := identity.GetServiceEndpoint("s3"); dynamicS3Endpoint != "" {
+		s3Url = "https://" + dynamicS3Endpoint
+	} else if strings.HasPrefix(region, s3util.ChinaRegionPrefix) {
+		s3Url = updateconstants.ChinaS3URL
+	} else {
+		s3Url = updateconstants.CommonS3URL
+	}
+
+	return strings.Replace(s3Url+updateconstants.BucketPath, updateconstants.RegionHolder, region, -1)
+}
+
 // IsV1UpdatePlugin returns true if source agent version is equal or below 3.0.882.0, any error defaults to false
-//  this logic is required since moving logic from plugin to updater would otherwise lead
-//  to duplicate aws console logging when upgrading from V1UpdatePlugin agents
+//
+//	this logic is required since moving logic from plugin to updater would otherwise lead
+//	to duplicate aws console logging when upgrading from V1UpdatePlugin agents
 func IsV1UpdatePlugin(SourceVersion string) bool {
 	const LastV1UpdatePluginAgentVersion = "3.0.882.0"
 
 	comp, err := versionutil.VersionCompare(SourceVersion, LastV1UpdatePluginAgentVersion)
 	return err == nil && comp <= 0
 }
+
+// IsIdentityRuntimeConfigSupported returns true if source agent version is greater than 3.1.282.0, any error defaults to false
+// this logic is required to ensure that updater does not rely on an old runtime config if an agent has been downgraded
+// from a version that supported identity runtimeconfig and also to skip the runtimeconfig check when we know it does not exist
+func IsIdentityRuntimeConfigSupported(sourceVersion string) bool {
+	const LastAgentVersionBeforeIdentityRuntimeConfig = "3.1.282.0"
+
+	comp, err := versionutil.VersionCompare(sourceVersion, LastAgentVersionBeforeIdentityRuntimeConfig)
+	return err == nil && comp > 0
+}
+
+func (util *Utility) setCommandEnvironmentVariables(command *exec.Cmd, commandEnv []string) {
+	credentialProvider, ok := getRemoteProvider(util.Context.Identity())
+	if !ok {
+		return
+	}
+
+	// Don't set environment variables if credentials are not being shared
+	if !credentialProvider.SharesCredentials() {
+		return
+	}
+
+	var osEnv []string
+	// set environment variables
+	if commandEnv != nil && len(commandEnv) > 0 {
+		// Update command env if set by the caller
+		// If not set, command passes the os.Environ() by default
+		osEnv = commandEnv
+	} else {
+		osEnv = os.Environ()
+	}
+
+	command.Env = osEnv
+	if _, ok := os.LookupEnv("AWS_SHARED_CREDENTIALS_FILE"); !ok && credentialProvider.ShareFile() != "" {
+		command.Env = append(command.Env, fmt.Sprintf("%s=%s", "AWS_SHARED_CREDENTIALS_FILE", credentialProvider.ShareFile()))
+	}
+
+	if _, ok := os.LookupEnv("AWS_PROFILE"); !ok && credentialProvider.ShareProfile() != "" {
+		command.Env = append(command.Env, fmt.Sprintf("%s=%s", "AWS_PROFILE", credentialProvider.ShareProfile()))
+	}
+
+}
