@@ -22,14 +22,17 @@ import (
 	"testing"
 	"time"
 
+	"github.com/aws/amazon-ssm-agent/agent/mocks/context"
+	"github.com/aws/amazon-ssm-agent/agent/mocks/log"
+	"github.com/aws/amazon-ssm-agent/common/identity"
+	"github.com/aws/amazon-ssm-agent/common/identity/credentialproviders"
+	"github.com/aws/amazon-ssm-agent/common/identity/credentialproviders/mocks"
+	identityMocks "github.com/aws/amazon-ssm-agent/common/identity/mocks"
+
 	"github.com/aws/amazon-ssm-agent/agent/appconfig"
 	"github.com/aws/amazon-ssm-agent/agent/fileutil"
-	"github.com/aws/amazon-ssm-agent/agent/log"
 	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateconstants"
-	"github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo"
-	updateinfomocks "github.com/aws/amazon-ssm-agent/agent/updateutil/updateinfo/mocks"
 	"github.com/aws/amazon-ssm-agent/core/executor"
-	executormocks "github.com/aws/amazon-ssm-agent/core/executor/mocks"
 	"github.com/aws/amazon-ssm-agent/core/workerprovider/longrunningprovider/model"
 	"github.com/stretchr/testify/assert"
 )
@@ -60,7 +63,11 @@ func TestCreateUpdateDownloadFolderSucceeded(t *testing.T) {
 	mkDirAll = func(path string, perm os.FileMode) error {
 		return nil
 	}
-	util := Utility{}
+
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
+
 	result, _ := util.CreateUpdateDownloadFolder()
 	assert.Contains(t, result, "update")
 }
@@ -69,7 +76,10 @@ func TestCreateUpdateDownloadFolderFailed(t *testing.T) {
 	mkDirAll = func(path string, perm os.FileMode) error {
 		return fmt.Errorf("Folder cannot be created")
 	}
-	util := Utility{}
+
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
 	_, err := util.CreateUpdateDownloadFolder()
 	assert.Error(t, err)
 }
@@ -92,6 +102,17 @@ func TestBuildUpdateCommand(t *testing.T) {
 	}
 }
 
+func TestConvertToUpdateErrorCode_MulipleCases(t *testing.T) {
+	output := ConvertToUpdateErrorCode(string(updateconstants.ErrorCreateUpdateFolder), "_", "Test")
+	assert.Equal(t, string(output), "ErrorCreateUpdateFolder_Test")
+	output = ConvertToUpdateErrorCode(string(updateconstants.ErrorCreateUpdateFolder))
+	assert.Equal(t, string(output), "ErrorCreateUpdateFolder")
+	output = ConvertToUpdateErrorCode("Test")
+	assert.Equal(t, string(output), "Test")
+	output = ConvertToUpdateErrorCode()
+	assert.Equal(t, string(output), "")
+}
+
 func TestUpdateArtifactFolder(t *testing.T) {
 	testCases := []struct {
 		pkgname string
@@ -206,92 +227,6 @@ func TestUnInstallerFilePath(t *testing.T) {
 	}
 }
 
-func TestIsServiceRunning(t *testing.T) {
-	infoRedHat65 := &updateinfomocks.T{}
-	infoRedHat65.On("GetPlatform").Return(updateconstants.PlatformRedHat)
-	infoRedHat65.On("IsPlatformUsingSystemD").Return(false, nil)
-	infoRedHat65.On("IsPlatformDarwin").Return(false)
-
-	infoRedHat71 := &updateinfomocks.T{}
-	infoRedHat71.On("GetPlatform").Return(updateconstants.PlatformRedHat)
-	infoRedHat71.On("IsPlatformUsingSystemD").Return(true, nil)
-	infoRedHat71.On("IsPlatformDarwin").Return(false)
-
-	infoDarwin := &updateinfomocks.T{}
-	infoDarwin.On("GetPlatform").Return(updateconstants.PlatformDarwin)
-	infoDarwin.On("IsPlatformUsingSystemD").Return(false, nil)
-	infoDarwin.On("IsPlatformDarwin").Return(true)
-
-	mock := &executormocks.IExecutor{}
-	mock.On("Processes").Return([]executor.OsProcess{{Executable: updateconstants.DarwinBinaryPath}}, nil)
-
-	util := Utility{
-		ProcessExecutor: mock,
-	}
-	testCases := []struct {
-		info   updateinfo.T
-		result bool
-	}{
-		// test system with upstart
-		{infoRedHat65, true},
-		// test system with systemD
-		{infoRedHat71, true},
-		// test system for mac os
-		{infoDarwin, true},
-	}
-
-	// Stub exec.Command
-	execCommand = fakeExecCommand
-
-	for _, test := range testCases {
-		fmt.Printf("Testing %s\n", test.info.GetPlatform())
-		result, _ := util.IsServiceRunning(logger, test.info)
-		assert.Equal(t, result, test.result)
-	}
-}
-
-func TestIsServiceRunningWithErrorMessageFromCommandExec(t *testing.T) {
-	infoRedHat65 := &updateinfomocks.T{}
-	infoRedHat65.On("GetPlatform").Return(updateconstants.PlatformRedHat)
-	infoRedHat65.On("IsPlatformUsingSystemD").Return(false, nil)
-	infoRedHat65.On("IsPlatformDarwin").Return(false)
-
-	infoRedHat71 := &updateinfomocks.T{}
-	infoRedHat71.On("GetPlatform").Return(updateconstants.PlatformRedHat)
-	infoRedHat71.On("IsPlatformUsingSystemD").Return(true, nil)
-	infoRedHat71.On("IsPlatformDarwin").Return(false)
-
-	infoDarwin := &updateinfomocks.T{}
-	infoDarwin.On("GetPlatform").Return(updateconstants.PlatformDarwin)
-	infoDarwin.On("IsPlatformUsingSystemD").Return(false, nil)
-	infoDarwin.On("IsPlatformDarwin").Return(true)
-
-	mock := &executormocks.IExecutor{}
-	mock.On("Processes").Return(nil, fmt.Errorf("SomeError"))
-	util := Utility{
-		ProcessExecutor: mock,
-	}
-	testCases := []struct {
-		info updateinfo.T
-	}{
-		// test system with upstart
-		{infoRedHat65},
-		// test system with systemD
-		{infoRedHat71},
-		// test system for mac os
-		{infoDarwin},
-	}
-
-	// Stub exec.Command
-	execCommand = fakeExecCommandWithError
-
-	for _, test := range testCases {
-		fmt.Printf("Testing %s\n", test.info.GetPlatform())
-		_, err := util.IsServiceRunning(logger, test.info)
-		assert.Error(t, err)
-	}
-}
-
 func TestExeCommandSucceeded(t *testing.T) {
 	testCases := []struct {
 		cmd            string
@@ -318,16 +253,21 @@ func TestExeCommandSucceeded(t *testing.T) {
 	execCommand = fakeExecCommand
 	cmdStart = func(*exec.Cmd) error { return nil }
 
-	util := Utility{}
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
 
 	for _, test := range testCases {
-		_, _, err := util.ExeCommand(logger,
-			test.cmd,
-			test.workingDir,
-			appconfig.UpdaterArtifactsRoot,
-			test.stdOut,
-			test.stdErr,
-			test.isAsync)
+		commandInput := CommandExecutionSettings{
+			Log:         logger,
+			Cmd:         strings.Fields(test.cmd),
+			WorkingDir:  test.workingDir,
+			UpdaterRoot: appconfig.UpdaterArtifactsRoot,
+			StdOut:      test.stdOut,
+			StdErr:      test.stdErr,
+			IsAsync:     test.isAsync,
+		}
+		_, _, err := util.ExeCommand(&commandInput)
 
 		if test.expectingError {
 			assert.Error(t, err)
@@ -430,7 +370,9 @@ func TestIsDiskSpaceSufficientForUpdateWithSufficientSpace(t *testing.T) {
 		}, nil
 	}
 
-	util := Utility{}
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
 	isSufficient, err := util.IsDiskSpaceSufficientForUpdate(logger)
 
 	assert.NoError(t, err)
@@ -446,7 +388,9 @@ func TestIsDiskSpaceSufficientForUpdateWithInsufficientSpace(t *testing.T) {
 		}, nil
 	}
 
-	util := Utility{}
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
 	isSufficient, err := util.IsDiskSpaceSufficientForUpdate(logger)
 
 	assert.NoError(t, err)
@@ -462,7 +406,9 @@ func TestIsDiskSpaceSufficientForUpdateWithDiskSpaceLoadFail(t *testing.T) {
 		}, fmt.Errorf("mock error - failed to load the disk space")
 	}
 
-	util := Utility{}
+	util := Utility{
+		Context: context.NewMockDefault(),
+	}
 	isSufficient, err := util.IsDiskSpaceSufficientForUpdate(logger)
 
 	assert.Error(t, err)
@@ -532,6 +478,41 @@ func TestCompareVersion(t *testing.T) {
 
 }
 
+func TestGetStableURLFromManifestURL(t *testing.T) {
+	// Empty URL
+	agentIdentity := identityMocks.NewDefaultMockAgentIdentity()
+	agentIdentity.On("Region").Return("us-east-1", nil)
+
+	url, err := GetStableURLFromManifestURL("", agentIdentity)
+	assert.Error(t, err)
+	assert.Equal(t, "", url)
+
+	// Invalid URL
+	url, err = GetStableURLFromManifestURL("InvalidUrl", agentIdentity)
+	assert.Error(t, err)
+	assert.Equal(t, "", url)
+
+	// Invalid manifest url - artifact url
+	url, err = GetStableURLFromManifestURL("https://bucket.s3.region.amazonaws.com/amazon-ssm-agent/version/amazon-ssm-agent.tar.gz", agentIdentity)
+	assert.Error(t, err)
+	assert.Equal(t, "", url)
+
+	// Prod manifest url with region placeholder
+	url, err = GetStableURLFromManifestURL("https://s3.{Region}.amazonaws.com/amazon-ssm-{Region}/ssm-agent-manifest.json", agentIdentity)
+	assert.NoError(t, err)
+	assert.Equal(t, "https://s3.us-east-1.amazonaws.com/amazon-ssm-us-east-1/stable/VERSION", url)
+
+	// Valid s3 manifest link bucket in Path
+	url, err = GetStableURLFromManifestURL("https://s3.region.amazonaws.com/bucket/ssm-agent-manifest.json", agentIdentity)
+	assert.NoError(t, err)
+	assert.Equal(t, "https://s3.region.amazonaws.com/bucket/stable/VERSION", url)
+
+	// Valid s3 manifest link with bucket in url
+	url, err = GetStableURLFromManifestURL("https://bucket.s3.region.amazonaws.com/ssm-agent-manifest.json", agentIdentity)
+	assert.NoError(t, err)
+	assert.Equal(t, "https://bucket.s3.region.amazonaws.com/stable/VERSION", url)
+}
+
 func TestGetManifestURLFromSourceUrl(t *testing.T) {
 	// Empty URL
 	url, err := GetManifestURLFromSourceUrl("")
@@ -583,6 +564,55 @@ func TestIsV1UpdatePlugin(t *testing.T) {
 	assert.False(t, IsV1UpdatePlugin("SomeInvalidVersion"))
 }
 
+func TestIsIdentityRuntimeConfigSupported(t *testing.T) {
+	// lower than
+	assert.False(t, IsIdentityRuntimeConfigSupported("3.0.281.0"))
+	assert.False(t, IsIdentityRuntimeConfigSupported("1.1.281.0"))
+	assert.False(t, IsIdentityRuntimeConfigSupported("3.1.281.0"))
+
+	// equal than
+	assert.False(t, IsIdentityRuntimeConfigSupported("3.1.282.0"))
+
+	// greater than
+	assert.True(t, IsIdentityRuntimeConfigSupported("3.1.282.1"))
+	assert.True(t, IsIdentityRuntimeConfigSupported("3.1.1111.0"))
+	assert.True(t, IsIdentityRuntimeConfigSupported("3.1.283.0"))
+	assert.True(t, IsIdentityRuntimeConfigSupported("3.2.283.0"))
+
+	// invalid source
+	assert.False(t, IsIdentityRuntimeConfigSupported("SomeInvalidVersion"))
+}
+
+func TestUtility_setShareCredsEnvironment_SetsCommandAWSEnvironmentVariables_WhenIdentitySharesCredentials(t *testing.T) {
+	ctx := new(context.Mock)
+	agentIdentity := identityMocks.NewDefaultMockAgentIdentity()
+	ctx.On("Identity").Return(agentIdentity)
+	ctx.On("Log").Return(log.NewMockLog())
+
+	remoteProvider := &mocks.IRemoteProvider{}
+	remoteProvider.On("SharesCredentials").Return(true)
+	expectedShareProfile := "SomeShareFileLocation"
+	expectedShareFile := "SomeShareFileLocation"
+	remoteProvider.On("ShareProfile").Return(expectedShareProfile)
+	remoteProvider.On("ShareFile").Return(expectedShareFile)
+	getRemoteProvider = func(agentIdentity identity.IAgentIdentity) (credentialproviders.IRemoteProvider, bool) {
+		return remoteProvider, true
+	}
+
+	utility := &Utility{
+		Context: ctx,
+	}
+
+	command := &exec.Cmd{}
+	utility.setCommandEnvironmentVariables(command, nil)
+
+	expectedProfileVar := fmt.Sprintf("AWS_PROFILE=%s", expectedShareProfile)
+	expectedSharedFileVar := fmt.Sprintf("AWS_SHARED_CREDENTIALS_FILE=%s", expectedShareFile)
+
+	assert.Contains(t, command.Env, expectedProfileVar, "AWS_PROFILE environment variable not set")
+	assert.Contains(t, command.Env, expectedSharedFileVar, "AWS_SHARED_CREDENTIALS_FILE environment variable not set")
+}
+
 func (p *testProcess) Start(*model.WorkerConfig) (*model.Process, error) { return nil, nil }
 
 func (p *testProcess) Kill(pid int) error { return nil }
